var vo=Object.defineProperty;var Fr=Object.getOwnPropertySymbols;var bo=Object.prototype.hasOwnProperty,No=Object.prototype.propertyIsEnumerable;var Ir=(t,n,e)=>n in t?vo(t,n,{enumerable:!0,configurable:!0,writable:!0,value:e}):t[n]=e,$e=(t,n)=>{for(var e in n||(n={}))bo.call(n,e)&&Ir(t,e,n[e]);if(Fr)for(var e of Fr(n))No.call(n,e)&&Ir(t,e,n[e]);return t};import{S as wi,i as pi,s as xi,J as vi,e as Re,k as ko,c as Ee,a as _r,m as Ao,d as vt,b as B,f as L,g as Ye,I as rr,R as j,L as bi,M as Ni,N as ki,q as pn,o as ze,G as So,a6 as Mo,v as Ai,Y as Do,t as Ro,h as Eo,E as zo,a4 as ir,W as Co,a7 as To,l as Lr,n as Po,p as Fo,a3 as _,a5 as p,r as Io}from"./index-d6b53ebe.js";/* empty css                                               */import{w as x,d as S}from"./index-e4f78b56.js";/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */function Nn(t,n,e,r){function i(o){return o instanceof e?o:new e(function(a){a(o)})}return new(e||(e=Promise))(function(o,a){function u(l){try{s(r.next(l))}catch(h){a(h)}}function f(l){try{s(r.throw(l))}catch(h){a(h)}}function s(l){l.done?o(l.value):i(l.value).then(u,f)}s((r=r.apply(t,n||[])).next())})}function kn(t,n){var e={label:0,sent:function(){if(o[0]&1)throw o[1];return o[1]},trys:[],ops:[]},r,i,o,a;return a={next:u(0),throw:u(1),return:u(2)},typeof Symbol=="function"&&(a[Symbol.iterator]=function(){return this}),a;function u(s){return function(l){return f([s,l])}}function f(s){if(r)throw new TypeError("Generator is already executing.");for(;e;)try{if(r=1,i&&(o=s[0]&2?i.return:s[0]?i.throw||((o=i.return)&&o.call(i),0):i.next)&&!(o=o.call(i,s[1])).done)return o;switch(i=0,o&&(s=[s[0]&2,o.value]),s[0]){case 0:case 1:o=s;break;case 4:return e.label++,{value:s[1],done:!1};case 5:e.label++,i=s[1],s=[0];continue;case 7:s=e.ops.pop(),e.trys.pop();continue;default:if(o=e.trys,!(o=o.length>0&&o[o.length-1])&&(s[0]===6||s[0]===2)){e=0;continue}if(s[0]===3&&(!o||s[1]>o[0]&&s[1]<o[3])){e.label=s[1];break}if(s[0]===6&&e.label<o[1]){e.label=o[1],o=s;break}if(o&&e.label<o[2]){e.label=o[2],e.ops.push(s);break}o[2]&&e.ops.pop(),e.trys.pop();continue}s=n.call(t,e)}catch(l){s=[6,l],i=0}finally{r=o=0}if(s[0]&5)throw s[1];return{value:s[0]?s[1]:void 0,done:!0}}}function Lo(t,n){var e=typeof Symbol=="function"&&t[Symbol.iterator];if(!e)return t;var r=e.call(t),i,o=[],a;try{for(;(n===void 0||n-- >0)&&!(i=r.next()).done;)o.push(i.value)}catch(u){a={error:u}}finally{try{i&&!i.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o}function Oo(){for(var t=[],n=0;n<arguments.length;n++)t=t.concat(Lo(arguments[n]));return t}var Ho=new Map([["avi","video/avi"],["gif","image/gif"],["ico","image/x-icon"],["jpeg","image/jpeg"],["jpg","image/jpeg"],["mkv","video/x-matroska"],["mov","video/quicktime"],["mp4","video/mp4"],["pdf","application/pdf"],["png","image/png"],["zip","application/zip"],["doc","application/msword"],["docx","application/vnd.openxmlformats-officedocument.wordprocessingml.document"]]);function Be(t,n){var e=qo(t);if(typeof e.path!="string"){var r=t.webkitRelativePath;Object.defineProperty(e,"path",{value:typeof n=="string"?n:typeof r=="string"&&r.length>0?r:t.name,writable:!1,configurable:!1,enumerable:!0})}return e}function qo(t){var n=t.name,e=n&&n.lastIndexOf(".")!==-1;if(e&&!t.type){var r=n.split(".").pop().toLowerCase(),i=Ho.get(r);i&&Object.defineProperty(t,"type",{value:i,writable:!1,configurable:!1,enumerable:!0})}return t}var Xo=[".DS_Store","Thumbs.db"];function Yo(t){return Nn(this,void 0,void 0,function(){return kn(this,function(n){return[2,Bo(t)&&t.dataTransfer?Wo(t.dataTransfer,t.type):Go(t)]})})}function Bo(t){return!!t.dataTransfer}function Go(t){var n=Vo(t.target)?t.target.files?or(t.target.files):[]:[];return n.map(function(e){return Be(e)})}function Vo(t){return t!==null}function Wo(t,n){return Nn(this,void 0,void 0,function(){var e,r;return kn(this,function(i){switch(i.label){case 0:return t.items?(e=or(t.items).filter(function(o){return o.kind==="file"}),n!=="drop"?[2,e]:[4,Promise.all(e.map(Ko))]):[3,2];case 1:return r=i.sent(),[2,Or(Si(r))];case 2:return[2,Or(or(t.files).map(function(o){return Be(o)}))]}})})}function Or(t){return t.filter(function(n){return Xo.indexOf(n.name)===-1})}function or(t){for(var n=[],e=0;e<t.length;e++){var r=t[e];n.push(r)}return n}function Ko(t){if(typeof t.webkitGetAsEntry!="function")return Hr(t);var n=t.webkitGetAsEntry();return n&&n.isDirectory?Mi(n):Hr(t)}function Si(t){return t.reduce(function(n,e){return Oo(n,Array.isArray(e)?Si(e):[e])},[])}function Hr(t){var n=t.getAsFile();if(!n)return Promise.reject(t+" is not a File");var e=Be(n);return Promise.resolve(e)}function Uo(t){return Nn(this,void 0,void 0,function(){return kn(this,function(n){return[2,t.isDirectory?Mi(t):Jo(t)]})})}function Mi(t){var n=t.createReader();return new Promise(function(e,r){var i=[];function o(){var a=this;n.readEntries(function(u){return Nn(a,void 0,void 0,function(){var f,s,l;return kn(this,function(h){switch(h.label){case 0:if(u.length)return[3,5];h.label=1;case 1:return h.trys.push([1,3,,4]),[4,Promise.all(i)];case 2:return f=h.sent(),e(f),[3,4];case 3:return s=h.sent(),r(s),[3,4];case 4:return[3,6];case 5:l=Promise.all(u.map(Uo)),i.push(l),o(),h.label=6;case 6:return[2]}})})},function(u){r(u)})}o()})}function Jo(t){return Nn(this,void 0,void 0,function(){return kn(this,function(n){return[2,new Promise(function(e,r){t.file(function(i){var o=Be(i,t.fullPath);e(o)},function(i){r(i)})})]})})}function Zo(t,n){if(t&&n){const e=Array.isArray(n)?n:n.split(","),r=t.name||"",i=(t.type||"").toLowerCase(),o=i.replace(/\/.*$/,"");return e.some(a=>{const u=a.trim().toLowerCase();return u.charAt(0)==="."?r.toLowerCase().endsWith(u):u.endsWith("/*")?o===u.replace(/\/.*$/,""):i===u})}return!0}const Qo="file-invalid-type",jo="file-too-large",$o="file-too-small",ta="too-many-files",na=t=>{t=Array.isArray(t)&&t.length===1?t[0]:t;const n=Array.isArray(t)?`one of ${t.join(", ")}`:t;return{code:Qo,message:`File type must be ${n}`}},qr=t=>({code:jo,message:`File is larger than ${t} bytes`}),Xr=t=>({code:$o,message:`File is smaller than ${t} bytes`}),ea={code:ta,message:"Too many files"};function ra(t,n){const e=t.type==="application/x-moz-file"||Zo(t,n);return[e,e?null:na(n)]}function ia(t,n,e){if(dn(t.size))if(dn(n)&&dn(e)){if(t.size>e)return[!1,qr(e)];if(t.size<n)return[!1,Xr(n)]}else{if(dn(n)&&t.size<n)return[!1,Xr(n)];if(dn(e)&&t.size>e)return[!1,qr(e)]}return[!0,null]}function dn(t){return t!=null}function Yr(t){return typeof t.isPropagationStopped=="function"?t.isPropagationStopped():typeof t.cancelBubble!="undefined"?t.cancelBubble:!1}function ge(t){return t.dataTransfer?Array.prototype.some.call(t.dataTransfer.types,n=>n==="Files"||n==="application/x-moz-file"):!!t.target&&!!t.target.files}function Br(t){t.preventDefault()}function oa(t){return t.indexOf("MSIE")!==-1||t.indexOf("Trident/")!==-1}function aa(t){return t.indexOf("Edge/")!==-1}function ua(t=window.navigator.userAgent){return oa(t)||aa(t)}function sa(t){let n,e;return{c(){n=Re("p"),e=Ro("Drag 'n' drop some files here, or click to select files")},l(r){n=Ee(r,"P",{});var i=_r(n);e=Eo(i,"Drag 'n' drop some files here, or click to select files"),i.forEach(vt)},m(r,i){Ye(r,n,i),rr(n,e)},p:zo,d(r){r&&vt(n)}}}function fa(t){let n,e,r,i,o,a,u;const f=t[28].default,s=vi(f,t,t[27],null),l=s||sa();return{c(){n=Re("div"),e=Re("input"),r=ko(),l&&l.c(),this.h()},l(h){n=Ee(h,"DIV",{tabindex:!0,class:!0,style:!0});var d=_r(n);e=Ee(d,"INPUT",{accept:!0,type:!0,autocomplete:!0,tabindex:!0,style:!0}),r=Ao(d),l&&l.l(d),d.forEach(vt),this.h()},h(){B(e,"accept",t[0]),e.multiple=t[1],B(e,"type","file"),B(e,"autocomplete","off"),B(e,"tabindex","-1"),L(e,"display","none"),B(n,"tabindex","0"),B(n,"class",i=(t[4]?"":"dropzone")+" "+t[2]+" svelte-4uigcm"),B(n,"style",t[3])},m(h,d){Ye(h,n,d),rr(n,e),t[29](e),rr(n,r),l&&l.m(n,null),t[30](n),o=!0,a||(u=[j(e,"change",t[14]),j(e,"click",ca),j(n,"keydown",t[16](t[7])),j(n,"focus",t[16](t[8])),j(n,"blur",t[16](t[9])),j(n,"click",t[15](t[10])),j(n,"dragenter",t[17](t[11])),j(n,"dragover",t[17](t[12])),j(n,"dragleave",t[17](t[13])),j(n,"drop",t[17](t[14]))],a=!0)},p(h,d){(!o||d[0]&1)&&B(e,"accept",h[0]),(!o||d[0]&2)&&(e.multiple=h[1]),s&&s.p&&(!o||d[0]&134217728)&&bi(s,f,h,h[27],o?ki(f,h[27],d,null):Ni(h[27]),null),(!o||d[0]&20&&i!==(i=(h[4]?"":"dropzone")+" "+h[2]+" svelte-4uigcm"))&&B(n,"class",i),(!o||d[0]&8)&&B(n,"style",h[3])},i(h){o||(pn(l,h),o=!0)},o(h){ze(l,h),o=!1},d(h){h&&vt(n),t[29](null),l&&l.d(h),t[30](null),a=!1,So(u)}}}function ca(t){t.stopPropagation()}function la(t,n,e){let{$$slots:r={},$$scope:i}=n,{accept:o}=n,{disabled:a=!1}=n,{getFilesFromEvent:u=Yo}=n,{maxSize:f=1/0}=n,{minSize:s=0}=n,{multiple:l=!0}=n,{preventDropOnDocument:h=!0}=n,{noClick:d=!1}=n,{noKeyboard:y=!1}=n,{noDrag:m=!1}=n,{noDragEventsBubbling:v=!1}=n,{containerClasses:M=""}=n,{containerStyles:T=""}=n,{disableDefaultStyles:E=!1}=n;const A=Mo();let k={isFocused:!1,isFileDialogActive:!1,isDragActive:!1,isDragAccept:!1,isDragReject:!1,draggedFiles:[],acceptedFiles:[],fileRejections:[]},b,z;function yt(){k.isFileDialogActive=!1,k.isDragActive=!1,k.draggedFiles=[],k.acceptedFiles=[],k.fileRejections=[]}function K(){z&&(e(6,z.value=null,z),k.isFileDialogActive=!0,z.click())}function mt(g){!b||!b.isEqualNode(g.target)||(g.keyCode===32||g.keyCode===13)&&(g.preventDefault(),K())}function ut(){k.isFocused=!0}function w(){k.isFocused=!1}function F(){d||(ua()?setTimeout(K,0):K())}function R(g){g.preventDefault(),ft(g),Z=[...Z,g.target],ge(g)&&Promise.resolve(u(g)).then(X=>{Yr(g)&&!v||(k.draggedFiles=X,k.isDragActive=!0,A("dragenter",{dragEvent:g}))})}function H(g){if(g.preventDefault(),ft(g),g.dataTransfer)try{g.dataTransfer.dropEffect="copy"}catch{}return ge(g)&&A("dragover",{dragEvent:g}),!1}function st(g){g.preventDefault(),ft(g);const X=Z.filter(Y=>b&&b.contains(Y)),I=X.indexOf(g.target);I!==-1&&X.splice(I,1),Z=X,!(X.length>0)&&(k.isDragActive=!1,k.draggedFiles=[],ge(g)&&A("dragleave",{dragEvent:g}))}function U(g){g.preventDefault(),ft(g),Z=[],ge(g)&&Promise.resolve(u(g)).then(X=>{if(Yr(g)&&!v)return;const I=[],Y=[];X.forEach(et=>{const[Dn,Rn]=ra(et,o),[kt,Zt]=ia(et,s,f);if(Dn&&kt)I.push(et);else{const Qt=[Rn,Zt].filter(ct=>ct);Y.push({file:et,errors:Qt})}}),!l&&I.length>1&&(I.forEach(et=>{Y.push({file:et,errors:[ea]})}),I.splice(0)),k.acceptedFiles=I,k.fileRejections=Y,A("drop",{acceptedFiles:I,fileRejections:Y,event:g}),Y.length>0&&A("droprejected",{fileRejections:Y,event:g}),I.length>0&&A("dropaccepted",{acceptedFiles:I,event:g})}),yt()}function q(g){return a?null:g}function J(g){return y?null:q(g)}function C(g){return m?null:q(g)}function ft(g){v&&g.stopPropagation()}let Z=[];function _t(g){b&&b.contains(g.target)||(g.preventDefault(),Z=[])}function wt(){k.isFileDialogActive&&setTimeout(()=>{if(z){const{files:g}=z;g.length||(k.isFileDialogActive=!1,A("filedialogcancel"))}},300)}Ai(()=>{window.addEventListener("focus",wt,!1),h&&(document.addEventListener("dragover",Br,!1),document.addEventListener("drop",_t,!1))}),Do(()=>{window.removeEventListener("focus",wt,!1),h&&(document.removeEventListener("dragover",Br),document.removeEventListener("drop",_t))});function Jt(g){ir[g?"unshift":"push"](()=>{z=g,e(6,z)})}function Mn(g){ir[g?"unshift":"push"](()=>{b=g,e(5,b)})}return t.$$set=g=>{"accept"in g&&e(0,o=g.accept),"disabled"in g&&e(18,a=g.disabled),"getFilesFromEvent"in g&&e(19,u=g.getFilesFromEvent),"maxSize"in g&&e(20,f=g.maxSize),"minSize"in g&&e(21,s=g.minSize),"multiple"in g&&e(1,l=g.multiple),"preventDropOnDocument"in g&&e(22,h=g.preventDropOnDocument),"noClick"in g&&e(23,d=g.noClick),"noKeyboard"in g&&e(24,y=g.noKeyboard),"noDrag"in g&&e(25,m=g.noDrag),"noDragEventsBubbling"in g&&e(26,v=g.noDragEventsBubbling),"containerClasses"in g&&e(2,M=g.containerClasses),"containerStyles"in g&&e(3,T=g.containerStyles),"disableDefaultStyles"in g&&e(4,E=g.disableDefaultStyles),"$$scope"in g&&e(27,i=g.$$scope)},[o,l,M,T,E,b,z,mt,ut,w,F,R,H,st,U,q,J,C,a,u,f,s,h,d,y,m,v,i,r,Jt,Mn]}class Tc extends wi{constructor(n){super(),pi(this,n,la,fa,xi,{accept:0,disabled:18,getFilesFromEvent:19,maxSize:20,minSize:21,multiple:1,preventDropOnDocument:22,noClick:23,noKeyboard:24,noDrag:25,noDragEventsBubbling:26,containerClasses:2,containerStyles:3,disableDefaultStyles:4},null,[-1,-1])}}function ar(t,n){return t==null||n==null?NaN:t<n?-1:t>n?1:t>=n?0:NaN}function Di(t){let n=t,e=t,r=t;t.length!==2&&(n=(u,f)=>t(u)-f,e=ar,r=(u,f)=>ar(t(u),f));function i(u,f,s=0,l=u.length){if(s<l){if(e(f,f)!==0)return l;do{const h=s+l>>>1;r(u[h],f)<0?s=h+1:l=h}while(s<l)}return s}function o(u,f,s=0,l=u.length){if(s<l){if(e(f,f)!==0)return l;do{const h=s+l>>>1;r(u[h],f)<=0?s=h+1:l=h}while(s<l)}return s}function a(u,f,s=0,l=u.length){const h=i(u,f,s,l-1);return h>s&&n(u[h-1],f)>-n(u[h],f)?h-1:h}return{left:i,center:a,right:o}}function ha(t){return t===null?NaN:+t}function*Pc(t,n){if(n===void 0)for(let e of t)e!=null&&(e=+e)>=e&&(yield e);else{let e=-1;for(let r of t)(r=n(r,++e,t))!=null&&(r=+r)>=r&&(yield r)}}const da=Di(ar),ga=da.right;Di(ha).center;var ya=ga;class Gr extends Map{constructor(n,e=wa){if(super(),Object.defineProperties(this,{_intern:{value:new Map},_key:{value:e}}),n!=null)for(const[r,i]of n)this.set(r,i)}get(n){return super.get(Vr(this,n))}has(n){return super.has(Vr(this,n))}set(n,e){return super.set(ma(this,n),e)}delete(n){return super.delete(_a(this,n))}}function Vr({_intern:t,_key:n},e){const r=n(e);return t.has(r)?t.get(r):e}function ma({_intern:t,_key:n},e){const r=n(e);return t.has(r)?t.get(r):(t.set(r,e),e)}function _a({_intern:t,_key:n},e){const r=n(e);return t.has(r)&&(e=t.get(r),t.delete(r)),e}function wa(t){return t!==null&&typeof t=="object"?t.valueOf():t}var ur=Math.sqrt(50),sr=Math.sqrt(10),fr=Math.sqrt(2);function pa(t,n,e){var r,i=-1,o,a,u;if(n=+n,t=+t,e=+e,t===n&&e>0)return[t];if((r=n<t)&&(o=t,t=n,n=o),(u=Ri(t,n,e))===0||!isFinite(u))return[];if(u>0){let f=Math.round(t/u),s=Math.round(n/u);for(f*u<t&&++f,s*u>n&&--s,a=new Array(o=s-f+1);++i<o;)a[i]=(f+i)*u}else{u=-u;let f=Math.round(t*u),s=Math.round(n*u);for(f/u<t&&++f,s/u>n&&--s,a=new Array(o=s-f+1);++i<o;)a[i]=(f+i)/u}return r&&a.reverse(),a}function Ri(t,n,e){var r=(n-t)/Math.max(0,e),i=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,i);return i>=0?(o>=ur?10:o>=sr?5:o>=fr?2:1)*Math.pow(10,i):-Math.pow(10,-i)/(o>=ur?10:o>=sr?5:o>=fr?2:1)}function xa(t,n,e){var r=Math.abs(n-t)/Math.max(0,e),i=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/i;return o>=ur?i*=10:o>=sr?i*=5:o>=fr&&(i*=2),n<t?-i:i}function wr(t,n){switch(arguments.length){case 0:break;case 1:this.range(t);break;default:this.range(n).domain(t);break}return this}function Fc(t,n){switch(arguments.length){case 0:break;case 1:{typeof t=="function"?this.interpolator(t):this.range(t);break}default:{this.domain(t),typeof n=="function"?this.interpolator(n):this.range(n);break}}return this}const Wr=Symbol("implicit");function va(){var t=new Gr,n=[],e=[],r=Wr;function i(o){let a=t.get(o);if(a===void 0){if(r!==Wr)return r;t.set(o,a=n.push(o)-1)}return e[a%e.length]}return i.domain=function(o){if(!arguments.length)return n.slice();n=[],t=new Gr;for(const a of o)t.has(a)||t.set(a,n.push(a)-1);return i},i.range=function(o){return arguments.length?(e=Array.from(o),i):e.slice()},i.unknown=function(o){return arguments.length?(r=o,i):r},i.copy=function(){return va(n,e).unknown(r)},wr.apply(i,arguments),i}function pr(t,n,e){t.prototype=n.prototype=e,e.constructor=t}function Ei(t,n){var e=Object.create(t.prototype);for(var r in n)e[r]=n[r];return e}function An(){}var xn=.7,Ce=1/xn,Vt="\\s*([+-]?\\d+)\\s*",vn="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*",tt="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*",ba=/^#([0-9a-f]{3,8})$/,Na=new RegExp(`^rgb\\(${Vt},${Vt},${Vt}\\)$`),ka=new RegExp(`^rgb\\(${tt},${tt},${tt}\\)$`),Aa=new RegExp(`^rgba\\(${Vt},${Vt},${Vt},${vn}\\)$`),Sa=new RegExp(`^rgba\\(${tt},${tt},${tt},${vn}\\)$`),Ma=new RegExp(`^hsl\\(${vn},${tt},${tt}\\)$`),Da=new RegExp(`^hsla\\(${vn},${tt},${tt},${vn}\\)$`),Kr={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};pr(An,bt,{copy(t){return Object.assign(new this.constructor,this,t)},displayable(){return this.rgb().displayable()},hex:Ur,formatHex:Ur,formatHex8:Ra,formatHsl:Ea,formatRgb:Jr,toString:Jr});function Ur(){return this.rgb().formatHex()}function Ra(){return this.rgb().formatHex8()}function Ea(){return zi(this).formatHsl()}function Jr(){return this.rgb().formatRgb()}function bt(t){var n,e;return t=(t+"").trim().toLowerCase(),(n=ba.exec(t))?(e=n[1].length,n=parseInt(n[1],16),e===6?Zr(n):e===3?new P(n>>8&15|n>>4&240,n>>4&15|n&240,(n&15)<<4|n&15,1):e===8?ye(n>>24&255,n>>16&255,n>>8&255,(n&255)/255):e===4?ye(n>>12&15|n>>8&240,n>>8&15|n>>4&240,n>>4&15|n&240,((n&15)<<4|n&15)/255):null):(n=Na.exec(t))?new P(n[1],n[2],n[3],1):(n=ka.exec(t))?new P(n[1]*255/100,n[2]*255/100,n[3]*255/100,1):(n=Aa.exec(t))?ye(n[1],n[2],n[3],n[4]):(n=Sa.exec(t))?ye(n[1]*255/100,n[2]*255/100,n[3]*255/100,n[4]):(n=Ma.exec(t))?$r(n[1],n[2]/100,n[3]/100,1):(n=Da.exec(t))?$r(n[1],n[2]/100,n[3]/100,n[4]):Kr.hasOwnProperty(t)?Zr(Kr[t]):t==="transparent"?new P(NaN,NaN,NaN,0):null}function Zr(t){return new P(t>>16&255,t>>8&255,t&255,1)}function ye(t,n,e,r){return r<=0&&(t=n=e=NaN),new P(t,n,e,r)}function za(t){return t instanceof An||(t=bt(t)),t?(t=t.rgb(),new P(t.r,t.g,t.b,t.opacity)):new P}function Te(t,n,e,r){return arguments.length===1?za(t):new P(t,n,e,r==null?1:r)}function P(t,n,e,r){this.r=+t,this.g=+n,this.b=+e,this.opacity=+r}pr(P,Te,Ei(An,{brighter(t){return t=t==null?Ce:Math.pow(Ce,t),new P(this.r*t,this.g*t,this.b*t,this.opacity)},darker(t){return t=t==null?xn:Math.pow(xn,t),new P(this.r*t,this.g*t,this.b*t,this.opacity)},rgb(){return this},clamp(){return new P(xt(this.r),xt(this.g),xt(this.b),Pe(this.opacity))},displayable(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:Qr,formatHex:Qr,formatHex8:Ca,formatRgb:jr,toString:jr}));function Qr(){return`#${pt(this.r)}${pt(this.g)}${pt(this.b)}`}function Ca(){return`#${pt(this.r)}${pt(this.g)}${pt(this.b)}${pt((isNaN(this.opacity)?1:this.opacity)*255)}`}function jr(){const t=Pe(this.opacity);return`${t===1?"rgb(":"rgba("}${xt(this.r)}, ${xt(this.g)}, ${xt(this.b)}${t===1?")":`, ${t})`}`}function Pe(t){return isNaN(t)?1:Math.max(0,Math.min(1,t))}function xt(t){return Math.max(0,Math.min(255,Math.round(t)||0))}function pt(t){return t=xt(t),(t<16?"0":"")+t.toString(16)}function $r(t,n,e,r){return r<=0?t=n=e=NaN:e<=0||e>=1?t=n=NaN:n<=0&&(t=NaN),new V(t,n,e,r)}function zi(t){if(t instanceof V)return new V(t.h,t.s,t.l,t.opacity);if(t instanceof An||(t=bt(t)),!t)return new V;if(t instanceof V)return t;t=t.rgb();var n=t.r/255,e=t.g/255,r=t.b/255,i=Math.min(n,e,r),o=Math.max(n,e,r),a=NaN,u=o-i,f=(o+i)/2;return u?(n===o?a=(e-r)/u+(e<r)*6:e===o?a=(r-n)/u+2:a=(n-e)/u+4,u/=f<.5?o+i:2-o-i,a*=60):u=f>0&&f<1?0:a,new V(a,u,f,t.opacity)}function Ta(t,n,e,r){return arguments.length===1?zi(t):new V(t,n,e,r==null?1:r)}function V(t,n,e,r){this.h=+t,this.s=+n,this.l=+e,this.opacity=+r}pr(V,Ta,Ei(An,{brighter(t){return t=t==null?Ce:Math.pow(Ce,t),new V(this.h,this.s,this.l*t,this.opacity)},darker(t){return t=t==null?xn:Math.pow(xn,t),new V(this.h,this.s,this.l*t,this.opacity)},rgb(){var t=this.h%360+(this.h<0)*360,n=isNaN(t)||isNaN(this.s)?0:this.s,e=this.l,r=e+(e<.5?e:1-e)*n,i=2*e-r;return new P(tr(t>=240?t-240:t+120,i,r),tr(t,i,r),tr(t<120?t+240:t-120,i,r),this.opacity)},clamp(){return new V(ti(this.h),me(this.s),me(this.l),Pe(this.opacity))},displayable(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1},formatHsl(){const t=Pe(this.opacity);return`${t===1?"hsl(":"hsla("}${ti(this.h)}, ${me(this.s)*100}%, ${me(this.l)*100}%${t===1?")":`, ${t})`}`}}));function ti(t){return t=(t||0)%360,t<0?t+360:t}function me(t){return Math.max(0,Math.min(1,t||0))}function tr(t,n,e){return(t<60?n+(e-n)*t/60:t<180?e:t<240?n+(e-n)*(240-t)/60:n)*255}function Pa(t,n,e,r,i){var o=t*t,a=o*t;return((1-3*t+3*o-a)*n+(4-6*o+3*a)*e+(1+3*t+3*o-3*a)*r+a*i)/6}function Fa(t){var n=t.length-1;return function(e){var r=e<=0?e=0:e>=1?(e=1,n-1):Math.floor(e*n),i=t[r],o=t[r+1],a=r>0?t[r-1]:2*i-o,u=r<n-1?t[r+2]:2*o-i;return Pa((e-r/n)*n,a,i,o,u)}}var xr=t=>()=>t;function Ia(t,n){return function(e){return t+e*n}}function La(t,n,e){return t=Math.pow(t,e),n=Math.pow(n,e)-t,e=1/e,function(r){return Math.pow(t+r*n,e)}}function Oa(t){return(t=+t)==1?Ci:function(n,e){return e-n?La(n,e,t):xr(isNaN(n)?e:n)}}function Ci(t,n){var e=n-t;return e?Ia(t,e):xr(isNaN(t)?n:t)}var Fe=function t(n){var e=Oa(n);function r(i,o){var a=e((i=Te(i)).r,(o=Te(o)).r),u=e(i.g,o.g),f=e(i.b,o.b),s=Ci(i.opacity,o.opacity);return function(l){return i.r=a(l),i.g=u(l),i.b=f(l),i.opacity=s(l),i+""}}return r.gamma=t,r}(1);function Ha(t){return function(n){var e=n.length,r=new Array(e),i=new Array(e),o=new Array(e),a,u;for(a=0;a<e;++a)u=Te(n[a]),r[a]=u.r||0,i[a]=u.g||0,o[a]=u.b||0;return r=t(r),i=t(i),o=t(o),u.opacity=1,function(f){return u.r=r(f),u.g=i(f),u.b=o(f),u+""}}}var Ic=Ha(Fa);function qa(t,n){n||(n=[]);var e=t?Math.min(n.length,t.length):0,r=n.slice(),i;return function(o){for(i=0;i<e;++i)r[i]=t[i]*(1-o)+n[i]*o;return r}}function Xa(t){return ArrayBuffer.isView(t)&&!(t instanceof DataView)}function Ya(t,n){var e=n?n.length:0,r=t?Math.min(e,t.length):0,i=new Array(r),o=new Array(e),a;for(a=0;a<r;++a)i[a]=vr(t[a],n[a]);for(;a<e;++a)o[a]=n[a];return function(u){for(a=0;a<r;++a)o[a]=i[a](u);return o}}function Ba(t,n){var e=new Date;return t=+t,n=+n,function(r){return e.setTime(t*(1-r)+n*r),e}}function G(t,n){return t=+t,n=+n,function(e){return t*(1-e)+n*e}}function Ga(t,n){var e={},r={},i;(t===null||typeof t!="object")&&(t={}),(n===null||typeof n!="object")&&(n={});for(i in n)i in t?e[i]=vr(t[i],n[i]):r[i]=n[i];return function(o){for(i in e)r[i]=e[i](o);return r}}var cr=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,nr=new RegExp(cr.source,"g");function Va(t){return function(){return t}}function Wa(t){return function(n){return t(n)+""}}function Ti(t,n){var e=cr.lastIndex=nr.lastIndex=0,r,i,o,a=-1,u=[],f=[];for(t=t+"",n=n+"";(r=cr.exec(t))&&(i=nr.exec(n));)(o=i.index)>e&&(o=n.slice(e,o),u[a]?u[a]+=o:u[++a]=o),(r=r[0])===(i=i[0])?u[a]?u[a]+=i:u[++a]=i:(u[++a]=null,f.push({i:a,x:G(r,i)})),e=nr.lastIndex;return e<n.length&&(o=n.slice(e),u[a]?u[a]+=o:u[++a]=o),u.length<2?f[0]?Wa(f[0].x):Va(n):(n=f.length,function(s){for(var l=0,h;l<n;++l)u[(h=f[l]).i]=h.x(s);return u.join("")})}function vr(t,n){var e=typeof n,r;return n==null||e==="boolean"?xr(n):(e==="number"?G:e==="string"?(r=bt(n))?(n=r,Fe):Ti:n instanceof bt?Fe:n instanceof Date?Ba:Xa(n)?qa:Array.isArray(n)?Ya:typeof n.valueOf!="function"&&typeof n.toString!="function"||isNaN(n)?Ga:G)(t,n)}function Ka(t,n){return t=+t,n=+n,function(e){return Math.round(t*(1-e)+n*e)}}var ni=180/Math.PI,lr={translateX:0,translateY:0,rotate:0,skewX:0,scaleX:1,scaleY:1};function Pi(t,n,e,r,i,o){var a,u,f;return(a=Math.sqrt(t*t+n*n))&&(t/=a,n/=a),(f=t*e+n*r)&&(e-=t*f,r-=n*f),(u=Math.sqrt(e*e+r*r))&&(e/=u,r/=u,f/=u),t*r<n*e&&(t=-t,n=-n,f=-f,a=-a),{translateX:i,translateY:o,rotate:Math.atan2(n,t)*ni,skewX:Math.atan(f)*ni,scaleX:a,scaleY:u}}var _e;function Ua(t){const n=new(typeof DOMMatrix=="function"?DOMMatrix:WebKitCSSMatrix)(t+"");return n.isIdentity?lr:Pi(n.a,n.b,n.c,n.d,n.e,n.f)}function Ja(t){return t==null||(_e||(_e=document.createElementNS("http://www.w3.org/2000/svg","g")),_e.setAttribute("transform",t),!(t=_e.transform.baseVal.consolidate()))?lr:(t=t.matrix,Pi(t.a,t.b,t.c,t.d,t.e,t.f))}function Fi(t,n,e,r){function i(s){return s.length?s.pop()+" ":""}function o(s,l,h,d,y,m){if(s!==h||l!==d){var v=y.push("translate(",null,n,null,e);m.push({i:v-4,x:G(s,h)},{i:v-2,x:G(l,d)})}else(h||d)&&y.push("translate("+h+n+d+e)}function a(s,l,h,d){s!==l?(s-l>180?l+=360:l-s>180&&(s+=360),d.push({i:h.push(i(h)+"rotate(",null,r)-2,x:G(s,l)})):l&&h.push(i(h)+"rotate("+l+r)}function u(s,l,h,d){s!==l?d.push({i:h.push(i(h)+"skewX(",null,r)-2,x:G(s,l)}):l&&h.push(i(h)+"skewX("+l+r)}function f(s,l,h,d,y,m){if(s!==h||l!==d){var v=y.push(i(y)+"scale(",null,",",null,")");m.push({i:v-4,x:G(s,h)},{i:v-2,x:G(l,d)})}else(h!==1||d!==1)&&y.push(i(y)+"scale("+h+","+d+")")}return function(s,l){var h=[],d=[];return s=t(s),l=t(l),o(s.translateX,s.translateY,l.translateX,l.translateY,h,d),a(s.rotate,l.rotate,h,d),u(s.skewX,l.skewX,h,d),f(s.scaleX,s.scaleY,l.scaleX,l.scaleY,h,d),s=l=null,function(y){for(var m=-1,v=d.length,M;++m<v;)h[(M=d[m]).i]=M.x(y);return h.join("")}}}var Za=Fi(Ua,"px, ","px)","deg)"),Qa=Fi(Ja,", ",")",")");function ja(t){return function(){return t}}function $a(t){return+t}var ei=[0,1];function $(t){return t}function hr(t,n){return(n-=t=+t)?function(e){return(e-t)/n}:ja(isNaN(n)?NaN:.5)}function tu(t,n){var e;return t>n&&(e=t,t=n,n=e),function(r){return Math.max(t,Math.min(n,r))}}function nu(t,n,e){var r=t[0],i=t[1],o=n[0],a=n[1];return i<r?(r=hr(i,r),o=e(a,o)):(r=hr(r,i),o=e(o,a)),function(u){return o(r(u))}}function eu(t,n,e){var r=Math.min(t.length,n.length)-1,i=new Array(r),o=new Array(r),a=-1;for(t[r]<t[0]&&(t=t.slice().reverse(),n=n.slice().reverse());++a<r;)i[a]=hr(t[a],t[a+1]),o[a]=e(n[a],n[a+1]);return function(u){var f=ya(t,u,1,r)-1;return o[f](i[f](u))}}function Ii(t,n){return n.domain(t.domain()).range(t.range()).interpolate(t.interpolate()).clamp(t.clamp()).unknown(t.unknown())}function Li(){var t=ei,n=ei,e=vr,r,i,o,a=$,u,f,s;function l(){var d=Math.min(t.length,n.length);return a!==$&&(a=tu(t[0],t[d-1])),u=d>2?eu:nu,f=s=null,h}function h(d){return d==null||isNaN(d=+d)?o:(f||(f=u(t.map(r),n,e)))(r(a(d)))}return h.invert=function(d){return a(i((s||(s=u(n,t.map(r),G)))(d)))},h.domain=function(d){return arguments.length?(t=Array.from(d,$a),l()):t.slice()},h.range=function(d){return arguments.length?(n=Array.from(d),l()):n.slice()},h.rangeRound=function(d){return n=Array.from(d),e=Ka,l()},h.clamp=function(d){return arguments.length?(a=d?!0:$,l()):a!==$},h.interpolate=function(d){return arguments.length?(e=d,l()):e},h.unknown=function(d){return arguments.length?(o=d,h):o},function(d,y){return r=d,i=y,l()}}function ru(){return Li()($,$)}function iu(t){return Math.abs(t=Math.round(t))>=1e21?t.toLocaleString("en").replace(/,/g,""):t.toString(10)}function Ie(t,n){if((e=(t=n?t.toExponential(n-1):t.toExponential()).indexOf("e"))<0)return null;var e,r=t.slice(0,e);return[r.length>1?r[0]+r.slice(2):r,+t.slice(e+1)]}function Wt(t){return t=Ie(Math.abs(t)),t?t[1]:NaN}function ou(t,n){return function(e,r){for(var i=e.length,o=[],a=0,u=t[0],f=0;i>0&&u>0&&(f+u+1>r&&(u=Math.max(1,r-f)),o.push(e.substring(i-=u,i+u)),!((f+=u+1)>r));)u=t[a=(a+1)%t.length];return o.reverse().join(n)}}function au(t){return function(n){return n.replace(/[0-9]/g,function(e){return t[+e]})}}var uu=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function Le(t){if(!(n=uu.exec(t)))throw new Error("invalid format: "+t);var n;return new br({fill:n[1],align:n[2],sign:n[3],symbol:n[4],zero:n[5],width:n[6],comma:n[7],precision:n[8]&&n[8].slice(1),trim:n[9],type:n[10]})}Le.prototype=br.prototype;function br(t){this.fill=t.fill===void 0?" ":t.fill+"",this.align=t.align===void 0?">":t.align+"",this.sign=t.sign===void 0?"-":t.sign+"",this.symbol=t.symbol===void 0?"":t.symbol+"",this.zero=!!t.zero,this.width=t.width===void 0?void 0:+t.width,this.comma=!!t.comma,this.precision=t.precision===void 0?void 0:+t.precision,this.trim=!!t.trim,this.type=t.type===void 0?"":t.type+""}br.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(this.width===void 0?"":Math.max(1,this.width|0))+(this.comma?",":"")+(this.precision===void 0?"":"."+Math.max(0,this.precision|0))+(this.trim?"~":"")+this.type};function su(t){t:for(var n=t.length,e=1,r=-1,i;e<n;++e)switch(t[e]){case".":r=i=e;break;case"0":r===0&&(r=e),i=e;break;default:if(!+t[e])break t;r>0&&(r=0);break}return r>0?t.slice(0,r)+t.slice(i+1):t}var Oi;function fu(t,n){var e=Ie(t,n);if(!e)return t+"";var r=e[0],i=e[1],o=i-(Oi=Math.max(-8,Math.min(8,Math.floor(i/3)))*3)+1,a=r.length;return o===a?r:o>a?r+new Array(o-a+1).join("0"):o>0?r.slice(0,o)+"."+r.slice(o):"0."+new Array(1-o).join("0")+Ie(t,Math.max(0,n+o-1))[0]}function ri(t,n){var e=Ie(t,n);if(!e)return t+"";var r=e[0],i=e[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")}var ii={"%":(t,n)=>(t*100).toFixed(n),b:t=>Math.round(t).toString(2),c:t=>t+"",d:iu,e:(t,n)=>t.toExponential(n),f:(t,n)=>t.toFixed(n),g:(t,n)=>t.toPrecision(n),o:t=>Math.round(t).toString(8),p:(t,n)=>ri(t*100,n),r:ri,s:fu,X:t=>Math.round(t).toString(16).toUpperCase(),x:t=>Math.round(t).toString(16)};function oi(t){return t}var ai=Array.prototype.map,ui=["y","z","a","f","p","n","\xB5","m","","k","M","G","T","P","E","Z","Y"];function cu(t){var n=t.grouping===void 0||t.thousands===void 0?oi:ou(ai.call(t.grouping,Number),t.thousands+""),e=t.currency===void 0?"":t.currency[0]+"",r=t.currency===void 0?"":t.currency[1]+"",i=t.decimal===void 0?".":t.decimal+"",o=t.numerals===void 0?oi:au(ai.call(t.numerals,String)),a=t.percent===void 0?"%":t.percent+"",u=t.minus===void 0?"\u2212":t.minus+"",f=t.nan===void 0?"NaN":t.nan+"";function s(h){h=Le(h);var d=h.fill,y=h.align,m=h.sign,v=h.symbol,M=h.zero,T=h.width,E=h.comma,A=h.precision,k=h.trim,b=h.type;b==="n"?(E=!0,b="g"):ii[b]||(A===void 0&&(A=12),k=!0,b="g"),(M||d==="0"&&y==="=")&&(M=!0,d="0",y="=");var z=v==="$"?e:v==="#"&&/[boxX]/.test(b)?"0"+b.toLowerCase():"",yt=v==="$"?r:/[%p]/.test(b)?a:"",K=ii[b],mt=/[defgprs%]/.test(b);A=A===void 0?6:/[gprs]/.test(b)?Math.max(1,Math.min(21,A)):Math.max(0,Math.min(20,A));function ut(w){var F=z,R=yt,H,st,U;if(b==="c")R=K(w)+R,w="";else{w=+w;var q=w<0||1/w<0;if(w=isNaN(w)?f:K(Math.abs(w),A),k&&(w=su(w)),q&&+w==0&&m!=="+"&&(q=!1),F=(q?m==="("?m:u:m==="-"||m==="("?"":m)+F,R=(b==="s"?ui[8+Oi/3]:"")+R+(q&&m==="("?")":""),mt){for(H=-1,st=w.length;++H<st;)if(U=w.charCodeAt(H),48>U||U>57){R=(U===46?i+w.slice(H+1):w.slice(H))+R,w=w.slice(0,H);break}}}E&&!M&&(w=n(w,1/0));var J=F.length+w.length+R.length,C=J<T?new Array(T-J+1).join(d):"";switch(E&&M&&(w=n(C+w,C.length?T-R.length:1/0),C=""),y){case"<":w=F+w+R+C;break;case"=":w=F+C+w+R;break;case"^":w=C.slice(0,J=C.length>>1)+F+w+R+C.slice(J);break;default:w=C+F+w+R;break}return o(w)}return ut.toString=function(){return h+""},ut}function l(h,d){var y=s((h=Le(h),h.type="f",h)),m=Math.max(-8,Math.min(8,Math.floor(Wt(d)/3)))*3,v=Math.pow(10,-m),M=ui[8+m/3];return function(T){return y(v*T)+M}}return{format:s,formatPrefix:l}}var we,Hi,qi;lu({thousands:",",grouping:[3],currency:["$",""]});function lu(t){return we=cu(t),Hi=we.format,qi=we.formatPrefix,we}function hu(t){return Math.max(0,-Wt(Math.abs(t)))}function du(t,n){return Math.max(0,Math.max(-8,Math.min(8,Math.floor(Wt(n)/3)))*3-Wt(Math.abs(t)))}function gu(t,n){return t=Math.abs(t),n=Math.abs(n)-t,Math.max(0,Wt(n)-Wt(t))+1}function yu(t,n,e,r){var i=xa(t,n,e),o;switch(r=Le(r==null?",f":r),r.type){case"s":{var a=Math.max(Math.abs(t),Math.abs(n));return r.precision==null&&!isNaN(o=du(i,a))&&(r.precision=o),qi(r,a)}case"":case"e":case"g":case"p":case"r":{r.precision==null&&!isNaN(o=gu(i,Math.max(Math.abs(t),Math.abs(n))))&&(r.precision=o-(r.type==="e"));break}case"f":case"%":{r.precision==null&&!isNaN(o=hu(i))&&(r.precision=o-(r.type==="%")*2);break}}return Hi(r)}function Xi(t){var n=t.domain;return t.ticks=function(e){var r=n();return pa(r[0],r[r.length-1],e==null?10:e)},t.tickFormat=function(e,r){var i=n();return yu(i[0],i[i.length-1],e==null?10:e,r)},t.nice=function(e){e==null&&(e=10);var r=n(),i=0,o=r.length-1,a=r[i],u=r[o],f,s,l=10;for(u<a&&(s=a,a=u,u=s,s=i,i=o,o=s);l-- >0;){if(s=Ri(a,u,e),s===f)return r[i]=a,r[o]=u,n(r);if(s>0)a=Math.floor(a/s)*s,u=Math.ceil(u/s)*s;else if(s<0)a=Math.ceil(a*s)/s,u=Math.floor(u*s)/s;else break;f=s}return t},t}function Ae(){var t=ru();return t.copy=function(){return Ii(t,Ae())},wr.apply(t,arguments),Xi(t)}function si(t){return function(n){return n<0?-Math.pow(-n,t):Math.pow(n,t)}}function mu(t){return t<0?-Math.sqrt(-t):Math.sqrt(t)}function _u(t){return t<0?-t*t:t*t}function wu(t){var n=t($,$),e=1;function r(){return e===1?t($,$):e===.5?t(mu,_u):t(si(e),si(1/e))}return n.exponent=function(i){return arguments.length?(e=+i,r()):e},Xi(n)}function Yi(){var t=wu(Li());return t.copy=function(){return Ii(t,Yi()).exponent(t.exponent())},wr.apply(t,arguments),t}function pu(){return Yi.apply(null,arguments).exponent(.5)}var xu={value:()=>{}};function Bi(){for(var t=0,n=arguments.length,e={},r;t<n;++t){if(!(r=arguments[t]+"")||r in e||/[\s.]/.test(r))throw new Error("illegal type: "+r);e[r]=[]}return new Se(e)}function Se(t){this._=t}function vu(t,n){return t.trim().split(/^|\s+/).map(function(e){var r="",i=e.indexOf(".");if(i>=0&&(r=e.slice(i+1),e=e.slice(0,i)),e&&!n.hasOwnProperty(e))throw new Error("unknown type: "+e);return{type:e,name:r}})}Se.prototype=Bi.prototype={constructor:Se,on:function(t,n){var e=this._,r=vu(t+"",e),i,o=-1,a=r.length;if(arguments.length<2){for(;++o<a;)if((i=(t=r[o]).type)&&(i=bu(e[i],t.name)))return i;return}if(n!=null&&typeof n!="function")throw new Error("invalid callback: "+n);for(;++o<a;)if(i=(t=r[o]).type)e[i]=fi(e[i],t.name,n);else if(n==null)for(i in e)e[i]=fi(e[i],t.name,null);return this},copy:function(){var t={},n=this._;for(var e in n)t[e]=n[e].slice();return new Se(t)},call:function(t,n){if((i=arguments.length-2)>0)for(var e=new Array(i),r=0,i,o;r<i;++r)e[r]=arguments[r+2];if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(o=this._[t],r=0,i=o.length;r<i;++r)o[r].value.apply(n,e)},apply:function(t,n,e){if(!this._.hasOwnProperty(t))throw new Error("unknown type: "+t);for(var r=this._[t],i=0,o=r.length;i<o;++i)r[i].value.apply(n,e)}};function bu(t,n){for(var e=0,r=t.length,i;e<r;++e)if((i=t[e]).name===n)return i.value}function fi(t,n,e){for(var r=0,i=t.length;r<i;++r)if(t[r].name===n){t[r]=xu,t=t.slice(0,r).concat(t.slice(r+1));break}return e!=null&&t.push({name:n,value:e}),t}var dr="http://www.w3.org/1999/xhtml",ci={svg:"http://www.w3.org/2000/svg",xhtml:dr,xlink:"http://www.w3.org/1999/xlink",xml:"http://www.w3.org/XML/1998/namespace",xmlns:"http://www.w3.org/2000/xmlns/"};function Ge(t){var n=t+="",e=n.indexOf(":");return e>=0&&(n=t.slice(0,e))!=="xmlns"&&(t=t.slice(e+1)),ci.hasOwnProperty(n)?{space:ci[n],local:t}:t}function Nu(t){return function(){var n=this.ownerDocument,e=this.namespaceURI;return e===dr&&n.documentElement.namespaceURI===dr?n.createElement(t):n.createElementNS(e,t)}}function ku(t){return function(){return this.ownerDocument.createElementNS(t.space,t.local)}}function Gi(t){var n=Ge(t);return(n.local?ku:Nu)(n)}function Au(){}function Nr(t){return t==null?Au:function(){return this.querySelector(t)}}function Su(t){typeof t!="function"&&(t=Nr(t));for(var n=this._groups,e=n.length,r=new Array(e),i=0;i<e;++i)for(var o=n[i],a=o.length,u=r[i]=new Array(a),f,s,l=0;l<a;++l)(f=o[l])&&(s=t.call(f,f.__data__,l,o))&&("__data__"in f&&(s.__data__=f.__data__),u[l]=s);return new O(r,this._parents)}function Mu(t){return t==null?[]:Array.isArray(t)?t:Array.from(t)}function Du(){return[]}function Vi(t){return t==null?Du:function(){return this.querySelectorAll(t)}}function Ru(t){return function(){return Mu(t.apply(this,arguments))}}function Eu(t){typeof t=="function"?t=Ru(t):t=Vi(t);for(var n=this._groups,e=n.length,r=[],i=[],o=0;o<e;++o)for(var a=n[o],u=a.length,f,s=0;s<u;++s)(f=a[s])&&(r.push(t.call(f,f.__data__,s,a)),i.push(f));return new O(r,i)}function Wi(t){return function(){return this.matches(t)}}function Ki(t){return function(n){return n.matches(t)}}var zu=Array.prototype.find;function Cu(t){return function(){return zu.call(this.children,t)}}function Tu(){return this.firstElementChild}function Pu(t){return this.select(t==null?Tu:Cu(typeof t=="function"?t:Ki(t)))}var Fu=Array.prototype.filter;function Iu(){return Array.from(this.children)}function Lu(t){return function(){return Fu.call(this.children,t)}}function Ou(t){return this.selectAll(t==null?Iu:Lu(typeof t=="function"?t:Ki(t)))}function Hu(t){typeof t!="function"&&(t=Wi(t));for(var n=this._groups,e=n.length,r=new Array(e),i=0;i<e;++i)for(var o=n[i],a=o.length,u=r[i]=[],f,s=0;s<a;++s)(f=o[s])&&t.call(f,f.__data__,s,o)&&u.push(f);return new O(r,this._parents)}function Ui(t){return new Array(t.length)}function qu(){return new O(this._enter||this._groups.map(Ui),this._parents)}function Oe(t,n){this.ownerDocument=t.ownerDocument,this.namespaceURI=t.namespaceURI,this._next=null,this._parent=t,this.__data__=n}Oe.prototype={constructor:Oe,appendChild:function(t){return this._parent.insertBefore(t,this._next)},insertBefore:function(t,n){return this._parent.insertBefore(t,n)},querySelector:function(t){return this._parent.querySelector(t)},querySelectorAll:function(t){return this._parent.querySelectorAll(t)}};function Xu(t){return function(){return t}}function Yu(t,n,e,r,i,o){for(var a=0,u,f=n.length,s=o.length;a<s;++a)(u=n[a])?(u.__data__=o[a],r[a]=u):e[a]=new Oe(t,o[a]);for(;a<f;++a)(u=n[a])&&(i[a]=u)}function Bu(t,n,e,r,i,o,a){var u,f,s=new Map,l=n.length,h=o.length,d=new Array(l),y;for(u=0;u<l;++u)(f=n[u])&&(d[u]=y=a.call(f,f.__data__,u,n)+"",s.has(y)?i[u]=f:s.set(y,f));for(u=0;u<h;++u)y=a.call(t,o[u],u,o)+"",(f=s.get(y))?(r[u]=f,f.__data__=o[u],s.delete(y)):e[u]=new Oe(t,o[u]);for(u=0;u<l;++u)(f=n[u])&&s.get(d[u])===f&&(i[u]=f)}function Gu(t){return t.__data__}function Vu(t,n){if(!arguments.length)return Array.from(this,Gu);var e=n?Bu:Yu,r=this._parents,i=this._groups;typeof t!="function"&&(t=Xu(t));for(var o=i.length,a=new Array(o),u=new Array(o),f=new Array(o),s=0;s<o;++s){var l=r[s],h=i[s],d=h.length,y=Wu(t.call(l,l&&l.__data__,s,r)),m=y.length,v=u[s]=new Array(m),M=a[s]=new Array(m),T=f[s]=new Array(d);e(l,h,v,M,T,y,n);for(var E=0,A=0,k,b;E<m;++E)if(k=v[E]){for(E>=A&&(A=E+1);!(b=M[A])&&++A<m;);k._next=b||null}}return a=new O(a,r),a._enter=u,a._exit=f,a}function Wu(t){return typeof t=="object"&&"length"in t?t:Array.from(t)}function Ku(){return new O(this._exit||this._groups.map(Ui),this._parents)}function Uu(t,n,e){var r=this.enter(),i=this,o=this.exit();return typeof t=="function"?(r=t(r),r&&(r=r.selection())):r=r.append(t+""),n!=null&&(i=n(i),i&&(i=i.selection())),e==null?o.remove():e(o),r&&i?r.merge(i).order():i}function Ju(t){for(var n=t.selection?t.selection():t,e=this._groups,r=n._groups,i=e.length,o=r.length,a=Math.min(i,o),u=new Array(i),f=0;f<a;++f)for(var s=e[f],l=r[f],h=s.length,d=u[f]=new Array(h),y,m=0;m<h;++m)(y=s[m]||l[m])&&(d[m]=y);for(;f<i;++f)u[f]=e[f];return new O(u,this._parents)}function Zu(){for(var t=this._groups,n=-1,e=t.length;++n<e;)for(var r=t[n],i=r.length-1,o=r[i],a;--i>=0;)(a=r[i])&&(o&&a.compareDocumentPosition(o)^4&&o.parentNode.insertBefore(a,o),o=a);return this}function Qu(t){t||(t=ju);function n(h,d){return h&&d?t(h.__data__,d.__data__):!h-!d}for(var e=this._groups,r=e.length,i=new Array(r),o=0;o<r;++o){for(var a=e[o],u=a.length,f=i[o]=new Array(u),s,l=0;l<u;++l)(s=a[l])&&(f[l]=s);f.sort(n)}return new O(i,this._parents).order()}function ju(t,n){return t<n?-1:t>n?1:t>=n?0:NaN}function $u(){var t=arguments[0];return arguments[0]=this,t.apply(null,arguments),this}function ts(){return Array.from(this)}function ns(){for(var t=this._groups,n=0,e=t.length;n<e;++n)for(var r=t[n],i=0,o=r.length;i<o;++i){var a=r[i];if(a)return a}return null}function es(){let t=0;for(const n of this)++t;return t}function rs(){return!this.node()}function is(t){for(var n=this._groups,e=0,r=n.length;e<r;++e)for(var i=n[e],o=0,a=i.length,u;o<a;++o)(u=i[o])&&t.call(u,u.__data__,o,i);return this}function os(t){return function(){this.removeAttribute(t)}}function as(t){return function(){this.removeAttributeNS(t.space,t.local)}}function us(t,n){return function(){this.setAttribute(t,n)}}function ss(t,n){return function(){this.setAttributeNS(t.space,t.local,n)}}function fs(t,n){return function(){var e=n.apply(this,arguments);e==null?this.removeAttribute(t):this.setAttribute(t,e)}}function cs(t,n){return function(){var e=n.apply(this,arguments);e==null?this.removeAttributeNS(t.space,t.local):this.setAttributeNS(t.space,t.local,e)}}function ls(t,n){var e=Ge(t);if(arguments.length<2){var r=this.node();return e.local?r.getAttributeNS(e.space,e.local):r.getAttribute(e)}return this.each((n==null?e.local?as:os:typeof n=="function"?e.local?cs:fs:e.local?ss:us)(e,n))}function Ji(t){return t.ownerDocument&&t.ownerDocument.defaultView||t.document&&t||t.defaultView}function hs(t){return function(){this.style.removeProperty(t)}}function ds(t,n,e){return function(){this.style.setProperty(t,n,e)}}function gs(t,n,e){return function(){var r=n.apply(this,arguments);r==null?this.style.removeProperty(t):this.style.setProperty(t,r,e)}}function ys(t,n,e){return arguments.length>1?this.each((n==null?hs:typeof n=="function"?gs:ds)(t,n,e==null?"":e)):Kt(this.node(),t)}function Kt(t,n){return t.style.getPropertyValue(n)||Ji(t).getComputedStyle(t,null).getPropertyValue(n)}function ms(t){return function(){delete this[t]}}function _s(t,n){return function(){this[t]=n}}function ws(t,n){return function(){var e=n.apply(this,arguments);e==null?delete this[t]:this[t]=e}}function ps(t,n){return arguments.length>1?this.each((n==null?ms:typeof n=="function"?ws:_s)(t,n)):this.node()[t]}function Zi(t){return t.trim().split(/^|\s+/)}function kr(t){return t.classList||new Qi(t)}function Qi(t){this._node=t,this._names=Zi(t.getAttribute("class")||"")}Qi.prototype={add:function(t){var n=this._names.indexOf(t);n<0&&(this._names.push(t),this._node.setAttribute("class",this._names.join(" ")))},remove:function(t){var n=this._names.indexOf(t);n>=0&&(this._names.splice(n,1),this._node.setAttribute("class",this._names.join(" ")))},contains:function(t){return this._names.indexOf(t)>=0}};function ji(t,n){for(var e=kr(t),r=-1,i=n.length;++r<i;)e.add(n[r])}function $i(t,n){for(var e=kr(t),r=-1,i=n.length;++r<i;)e.remove(n[r])}function xs(t){return function(){ji(this,t)}}function vs(t){return function(){$i(this,t)}}function bs(t,n){return function(){(n.apply(this,arguments)?ji:$i)(this,t)}}function Ns(t,n){var e=Zi(t+"");if(arguments.length<2){for(var r=kr(this.node()),i=-1,o=e.length;++i<o;)if(!r.contains(e[i]))return!1;return!0}return this.each((typeof n=="function"?bs:n?xs:vs)(e,n))}function ks(){this.textContent=""}function As(t){return function(){this.textContent=t}}function Ss(t){return function(){var n=t.apply(this,arguments);this.textContent=n==null?"":n}}function Ms(t){return arguments.length?this.each(t==null?ks:(typeof t=="function"?Ss:As)(t)):this.node().textContent}function Ds(){this.innerHTML=""}function Rs(t){return function(){this.innerHTML=t}}function Es(t){return function(){var n=t.apply(this,arguments);this.innerHTML=n==null?"":n}}function zs(t){return arguments.length?this.each(t==null?Ds:(typeof t=="function"?Es:Rs)(t)):this.node().innerHTML}function Cs(){this.nextSibling&&this.parentNode.appendChild(this)}function Ts(){return this.each(Cs)}function Ps(){this.previousSibling&&this.parentNode.insertBefore(this,this.parentNode.firstChild)}function Fs(){return this.each(Ps)}function Is(t){var n=typeof t=="function"?t:Gi(t);return this.select(function(){return this.appendChild(n.apply(this,arguments))})}function Ls(){return null}function Os(t,n){var e=typeof t=="function"?t:Gi(t),r=n==null?Ls:typeof n=="function"?n:Nr(n);return this.select(function(){return this.insertBefore(e.apply(this,arguments),r.apply(this,arguments)||null)})}function Hs(){var t=this.parentNode;t&&t.removeChild(this)}function qs(){return this.each(Hs)}function Xs(){var t=this.cloneNode(!1),n=this.parentNode;return n?n.insertBefore(t,this.nextSibling):t}function Ys(){var t=this.cloneNode(!0),n=this.parentNode;return n?n.insertBefore(t,this.nextSibling):t}function Bs(t){return this.select(t?Ys:Xs)}function Gs(t){return arguments.length?this.property("__data__",t):this.node().__data__}function Vs(t){return function(n){t.call(this,n,this.__data__)}}function Ws(t){return t.trim().split(/^|\s+/).map(function(n){var e="",r=n.indexOf(".");return r>=0&&(e=n.slice(r+1),n=n.slice(0,r)),{type:n,name:e}})}function Ks(t){return function(){var n=this.__on;if(!!n){for(var e=0,r=-1,i=n.length,o;e<i;++e)o=n[e],(!t.type||o.type===t.type)&&o.name===t.name?this.removeEventListener(o.type,o.listener,o.options):n[++r]=o;++r?n.length=r:delete this.__on}}}function Us(t,n,e){return function(){var r=this.__on,i,o=Vs(n);if(r){for(var a=0,u=r.length;a<u;++a)if((i=r[a]).type===t.type&&i.name===t.name){this.removeEventListener(i.type,i.listener,i.options),this.addEventListener(i.type,i.listener=o,i.options=e),i.value=n;return}}this.addEventListener(t.type,o,e),i={type:t.type,name:t.name,value:n,listener:o,options:e},r?r.push(i):this.__on=[i]}}function Js(t,n,e){var r=Ws(t+""),i,o=r.length,a;if(arguments.length<2){var u=this.node().__on;if(u){for(var f=0,s=u.length,l;f<s;++f)for(i=0,l=u[f];i<o;++i)if((a=r[i]).type===l.type&&a.name===l.name)return l.value}return}for(u=n?Us:Ks,i=0;i<o;++i)this.each(u(r[i],n,e));return this}function to(t,n,e){var r=Ji(t),i=r.CustomEvent;typeof i=="function"?i=new i(n,e):(i=r.document.createEvent("Event"),e?(i.initEvent(n,e.bubbles,e.cancelable),i.detail=e.detail):i.initEvent(n,!1,!1)),t.dispatchEvent(i)}function Zs(t,n){return function(){return to(this,t,n)}}function Qs(t,n){return function(){return to(this,t,n.apply(this,arguments))}}function js(t,n){return this.each((typeof n=="function"?Qs:Zs)(t,n))}function*$s(){for(var t=this._groups,n=0,e=t.length;n<e;++n)for(var r=t[n],i=0,o=r.length,a;i<o;++i)(a=r[i])&&(yield a)}var no=[null];function O(t,n){this._groups=t,this._parents=n}function Sn(){return new O([[document.documentElement]],no)}function tf(){return this}O.prototype=Sn.prototype={constructor:O,select:Su,selectAll:Eu,selectChild:Pu,selectChildren:Ou,filter:Hu,data:Vu,enter:qu,exit:Ku,join:Uu,merge:Ju,selection:tf,order:Zu,sort:Qu,call:$u,nodes:ts,node:ns,size:es,empty:rs,each:is,attr:ls,style:ys,property:ps,classed:Ns,text:Ms,html:zs,raise:Ts,lower:Fs,append:Is,insert:Os,remove:qs,clone:Bs,datum:Gs,on:Js,dispatch:js,[Symbol.iterator]:$s};function Lc(t){return typeof t=="string"?new O([[document.querySelector(t)]],[document.documentElement]):new O([[t]],no)}var Ut=0,yn=0,gn=0,eo=1e3,He,mn,qe=0,Nt=0,Ve=0,bn=typeof performance=="object"&&performance.now?performance:Date,ro=typeof window=="object"&&window.requestAnimationFrame?window.requestAnimationFrame.bind(window):function(t){setTimeout(t,17)};function Ar(){return Nt||(ro(nf),Nt=bn.now()+Ve)}function nf(){Nt=0}function Xe(){this._call=this._time=this._next=null}Xe.prototype=io.prototype={constructor:Xe,restart:function(t,n,e){if(typeof t!="function")throw new TypeError("callback is not a function");e=(e==null?Ar():+e)+(n==null?0:+n),!this._next&&mn!==this&&(mn?mn._next=this:He=this,mn=this),this._call=t,this._time=e,gr()},stop:function(){this._call&&(this._call=null,this._time=1/0,gr())}};function io(t,n,e){var r=new Xe;return r.restart(t,n,e),r}function ef(){Ar(),++Ut;for(var t=He,n;t;)(n=Nt-t._time)>=0&&t._call.call(void 0,n),t=t._next;--Ut}function li(){Nt=(qe=bn.now())+Ve,Ut=yn=0;try{ef()}finally{Ut=0,of(),Nt=0}}function rf(){var t=bn.now(),n=t-qe;n>eo&&(Ve-=n,qe=t)}function of(){for(var t,n=He,e,r=1/0;n;)n._call?(r>n._time&&(r=n._time),t=n,n=n._next):(e=n._next,n._next=null,n=t?t._next=e:He=e);mn=t,gr(r)}function gr(t){if(!Ut){yn&&(yn=clearTimeout(yn));var n=t-Nt;n>24?(t<1/0&&(yn=setTimeout(li,t-bn.now()-Ve)),gn&&(gn=clearInterval(gn))):(gn||(qe=bn.now(),gn=setInterval(rf,eo)),Ut=1,ro(li))}}function hi(t,n,e){var r=new Xe;return n=n==null?0:+n,r.restart(i=>{r.stop(),t(i+n)},n,e),r}var af=Bi("start","end","cancel","interrupt"),uf=[],oo=0,di=1,yr=2,Me=3,gi=4,mr=5,De=6;function We(t,n,e,r,i,o){var a=t.__transition;if(!a)t.__transition={};else if(e in a)return;sf(t,e,{name:n,index:r,group:i,on:af,tween:uf,time:o.time,delay:o.delay,duration:o.duration,ease:o.ease,timer:null,state:oo})}function Sr(t,n){var e=W(t,n);if(e.state>oo)throw new Error("too late; already scheduled");return e}function nt(t,n){var e=W(t,n);if(e.state>Me)throw new Error("too late; already running");return e}function W(t,n){var e=t.__transition;if(!e||!(e=e[n]))throw new Error("transition not found");return e}function sf(t,n,e){var r=t.__transition,i;r[n]=e,e.timer=io(o,0,e.time);function o(s){e.state=di,e.timer.restart(a,e.delay,e.time),e.delay<=s&&a(s-e.delay)}function a(s){var l,h,d,y;if(e.state!==di)return f();for(l in r)if(y=r[l],y.name===e.name){if(y.state===Me)return hi(a);y.state===gi?(y.state=De,y.timer.stop(),y.on.call("interrupt",t,t.__data__,y.index,y.group),delete r[l]):+l<n&&(y.state=De,y.timer.stop(),y.on.call("cancel",t,t.__data__,y.index,y.group),delete r[l])}if(hi(function(){e.state===Me&&(e.state=gi,e.timer.restart(u,e.delay,e.time),u(s))}),e.state=yr,e.on.call("start",t,t.__data__,e.index,e.group),e.state===yr){for(e.state=Me,i=new Array(d=e.tween.length),l=0,h=-1;l<d;++l)(y=e.tween[l].value.call(t,t.__data__,e.index,e.group))&&(i[++h]=y);i.length=h+1}}function u(s){for(var l=s<e.duration?e.ease.call(null,s/e.duration):(e.timer.restart(f),e.state=mr,1),h=-1,d=i.length;++h<d;)i[h].call(t,l);e.state===mr&&(e.on.call("end",t,t.__data__,e.index,e.group),f())}function f(){e.state=De,e.timer.stop(),delete r[n];for(var s in r)return;delete t.__transition}}function ff(t,n){var e=t.__transition,r,i,o=!0,a;if(!!e){n=n==null?null:n+"";for(a in e){if((r=e[a]).name!==n){o=!1;continue}i=r.state>yr&&r.state<mr,r.state=De,r.timer.stop(),r.on.call(i?"interrupt":"cancel",t,t.__data__,r.index,r.group),delete e[a]}o&&delete t.__transition}}function cf(t){return this.each(function(){ff(this,t)})}function lf(t,n){var e,r;return function(){var i=nt(this,t),o=i.tween;if(o!==e){r=e=o;for(var a=0,u=r.length;a<u;++a)if(r[a].name===n){r=r.slice(),r.splice(a,1);break}}i.tween=r}}function hf(t,n,e){var r,i;if(typeof e!="function")throw new Error;return function(){var o=nt(this,t),a=o.tween;if(a!==r){i=(r=a).slice();for(var u={name:n,value:e},f=0,s=i.length;f<s;++f)if(i[f].name===n){i[f]=u;break}f===s&&i.push(u)}o.tween=i}}function df(t,n){var e=this._id;if(t+="",arguments.length<2){for(var r=W(this.node(),e).tween,i=0,o=r.length,a;i<o;++i)if((a=r[i]).name===t)return a.value;return null}return this.each((n==null?lf:hf)(e,t,n))}function Mr(t,n,e){var r=t._id;return t.each(function(){var i=nt(this,r);(i.value||(i.value={}))[n]=e.apply(this,arguments)}),function(i){return W(i,r).value[n]}}function ao(t,n){var e;return(typeof n=="number"?G:n instanceof bt?Fe:(e=bt(n))?(n=e,Fe):Ti)(t,n)}function gf(t){return function(){this.removeAttribute(t)}}function yf(t){return function(){this.removeAttributeNS(t.space,t.local)}}function mf(t,n,e){var r,i=e+"",o;return function(){var a=this.getAttribute(t);return a===i?null:a===r?o:o=n(r=a,e)}}function _f(t,n,e){var r,i=e+"",o;return function(){var a=this.getAttributeNS(t.space,t.local);return a===i?null:a===r?o:o=n(r=a,e)}}function wf(t,n,e){var r,i,o;return function(){var a,u=e(this),f;return u==null?void this.removeAttribute(t):(a=this.getAttribute(t),f=u+"",a===f?null:a===r&&f===i?o:(i=f,o=n(r=a,u)))}}function pf(t,n,e){var r,i,o;return function(){var a,u=e(this),f;return u==null?void this.removeAttributeNS(t.space,t.local):(a=this.getAttributeNS(t.space,t.local),f=u+"",a===f?null:a===r&&f===i?o:(i=f,o=n(r=a,u)))}}function xf(t,n){var e=Ge(t),r=e==="transform"?Qa:ao;return this.attrTween(t,typeof n=="function"?(e.local?pf:wf)(e,r,Mr(this,"attr."+t,n)):n==null?(e.local?yf:gf)(e):(e.local?_f:mf)(e,r,n))}function vf(t,n){return function(e){this.setAttribute(t,n.call(this,e))}}function bf(t,n){return function(e){this.setAttributeNS(t.space,t.local,n.call(this,e))}}function Nf(t,n){var e,r;function i(){var o=n.apply(this,arguments);return o!==r&&(e=(r=o)&&bf(t,o)),e}return i._value=n,i}function kf(t,n){var e,r;function i(){var o=n.apply(this,arguments);return o!==r&&(e=(r=o)&&vf(t,o)),e}return i._value=n,i}function Af(t,n){var e="attr."+t;if(arguments.length<2)return(e=this.tween(e))&&e._value;if(n==null)return this.tween(e,null);if(typeof n!="function")throw new Error;var r=Ge(t);return this.tween(e,(r.local?Nf:kf)(r,n))}function Sf(t,n){return function(){Sr(this,t).delay=+n.apply(this,arguments)}}function Mf(t,n){return n=+n,function(){Sr(this,t).delay=n}}function Df(t){var n=this._id;return arguments.length?this.each((typeof t=="function"?Sf:Mf)(n,t)):W(this.node(),n).delay}function Rf(t,n){return function(){nt(this,t).duration=+n.apply(this,arguments)}}function Ef(t,n){return n=+n,function(){nt(this,t).duration=n}}function zf(t){var n=this._id;return arguments.length?this.each((typeof t=="function"?Rf:Ef)(n,t)):W(this.node(),n).duration}function Cf(t,n){if(typeof n!="function")throw new Error;return function(){nt(this,t).ease=n}}function Tf(t){var n=this._id;return arguments.length?this.each(Cf(n,t)):W(this.node(),n).ease}function Pf(t,n){return function(){var e=n.apply(this,arguments);if(typeof e!="function")throw new Error;nt(this,t).ease=e}}function Ff(t){if(typeof t!="function")throw new Error;return this.each(Pf(this._id,t))}function If(t){typeof t!="function"&&(t=Wi(t));for(var n=this._groups,e=n.length,r=new Array(e),i=0;i<e;++i)for(var o=n[i],a=o.length,u=r[i]=[],f,s=0;s<a;++s)(f=o[s])&&t.call(f,f.__data__,s,o)&&u.push(f);return new at(r,this._parents,this._name,this._id)}function Lf(t){if(t._id!==this._id)throw new Error;for(var n=this._groups,e=t._groups,r=n.length,i=e.length,o=Math.min(r,i),a=new Array(r),u=0;u<o;++u)for(var f=n[u],s=e[u],l=f.length,h=a[u]=new Array(l),d,y=0;y<l;++y)(d=f[y]||s[y])&&(h[y]=d);for(;u<r;++u)a[u]=n[u];return new at(a,this._parents,this._name,this._id)}function Of(t){return(t+"").trim().split(/^|\s+/).every(function(n){var e=n.indexOf(".");return e>=0&&(n=n.slice(0,e)),!n||n==="start"})}function Hf(t,n,e){var r,i,o=Of(n)?Sr:nt;return function(){var a=o(this,t),u=a.on;u!==r&&(i=(r=u).copy()).on(n,e),a.on=i}}function qf(t,n){var e=this._id;return arguments.length<2?W(this.node(),e).on.on(t):this.each(Hf(e,t,n))}function Xf(t){return function(){var n=this.parentNode;for(var e in this.__transition)if(+e!==t)return;n&&n.removeChild(this)}}function Yf(){return this.on("end.remove",Xf(this._id))}function Bf(t){var n=this._name,e=this._id;typeof t!="function"&&(t=Nr(t));for(var r=this._groups,i=r.length,o=new Array(i),a=0;a<i;++a)for(var u=r[a],f=u.length,s=o[a]=new Array(f),l,h,d=0;d<f;++d)(l=u[d])&&(h=t.call(l,l.__data__,d,u))&&("__data__"in l&&(h.__data__=l.__data__),s[d]=h,We(s[d],n,e,d,s,W(l,e)));return new at(o,this._parents,n,e)}function Gf(t){var n=this._name,e=this._id;typeof t!="function"&&(t=Vi(t));for(var r=this._groups,i=r.length,o=[],a=[],u=0;u<i;++u)for(var f=r[u],s=f.length,l,h=0;h<s;++h)if(l=f[h]){for(var d=t.call(l,l.__data__,h,f),y,m=W(l,e),v=0,M=d.length;v<M;++v)(y=d[v])&&We(y,n,e,v,d,m);o.push(d),a.push(l)}return new at(o,a,n,e)}var Vf=Sn.prototype.constructor;function Wf(){return new Vf(this._groups,this._parents)}function Kf(t,n){var e,r,i;return function(){var o=Kt(this,t),a=(this.style.removeProperty(t),Kt(this,t));return o===a?null:o===e&&a===r?i:i=n(e=o,r=a)}}function uo(t){return function(){this.style.removeProperty(t)}}function Uf(t,n,e){var r,i=e+"",o;return function(){var a=Kt(this,t);return a===i?null:a===r?o:o=n(r=a,e)}}function Jf(t,n,e){var r,i,o;return function(){var a=Kt(this,t),u=e(this),f=u+"";return u==null&&(f=u=(this.style.removeProperty(t),Kt(this,t))),a===f?null:a===r&&f===i?o:(i=f,o=n(r=a,u))}}function Zf(t,n){var e,r,i,o="style."+n,a="end."+o,u;return function(){var f=nt(this,t),s=f.on,l=f.value[o]==null?u||(u=uo(n)):void 0;(s!==e||i!==l)&&(r=(e=s).copy()).on(a,i=l),f.on=r}}function Qf(t,n,e){var r=(t+="")=="transform"?Za:ao;return n==null?this.styleTween(t,Kf(t,r)).on("end.style."+t,uo(t)):typeof n=="function"?this.styleTween(t,Jf(t,r,Mr(this,"style."+t,n))).each(Zf(this._id,t)):this.styleTween(t,Uf(t,r,n),e).on("end.style."+t,null)}function jf(t,n,e){return function(r){this.style.setProperty(t,n.call(this,r),e)}}function $f(t,n,e){var r,i;function o(){var a=n.apply(this,arguments);return a!==i&&(r=(i=a)&&jf(t,a,e)),r}return o._value=n,o}function tc(t,n,e){var r="style."+(t+="");if(arguments.length<2)return(r=this.tween(r))&&r._value;if(n==null)return this.tween(r,null);if(typeof n!="function")throw new Error;return this.tween(r,$f(t,n,e==null?"":e))}function nc(t){return function(){this.textContent=t}}function ec(t){return function(){var n=t(this);this.textContent=n==null?"":n}}function rc(t){return this.tween("text",typeof t=="function"?ec(Mr(this,"text",t)):nc(t==null?"":t+""))}function ic(t){return function(n){this.textContent=t.call(this,n)}}function oc(t){var n,e;function r(){var i=t.apply(this,arguments);return i!==e&&(n=(e=i)&&ic(i)),n}return r._value=t,r}function ac(t){var n="text";if(arguments.length<1)return(n=this.tween(n))&&n._value;if(t==null)return this.tween(n,null);if(typeof t!="function")throw new Error;return this.tween(n,oc(t))}function uc(){for(var t=this._name,n=this._id,e=so(),r=this._groups,i=r.length,o=0;o<i;++o)for(var a=r[o],u=a.length,f,s=0;s<u;++s)if(f=a[s]){var l=W(f,n);We(f,t,e,s,a,{time:l.time+l.delay+l.duration,delay:0,duration:l.duration,ease:l.ease})}return new at(r,this._parents,t,e)}function sc(){var t,n,e=this,r=e._id,i=e.size();return new Promise(function(o,a){var u={value:a},f={value:function(){--i===0&&o()}};e.each(function(){var s=nt(this,r),l=s.on;l!==t&&(n=(t=l).copy(),n._.cancel.push(u),n._.interrupt.push(u),n._.end.push(f)),s.on=n}),i===0&&o()})}var fc=0;function at(t,n,e,r){this._groups=t,this._parents=n,this._name=e,this._id=r}function so(){return++fc}var ot=Sn.prototype;at.prototype={constructor:at,select:Bf,selectAll:Gf,selectChild:ot.selectChild,selectChildren:ot.selectChildren,filter:If,merge:Lf,selection:Wf,transition:uc,call:ot.call,nodes:ot.nodes,node:ot.node,size:ot.size,empty:ot.empty,each:ot.each,on:qf,attr:xf,attrTween:Af,style:Qf,styleTween:tc,text:rc,textTween:ac,remove:Yf,tween:df,delay:Df,duration:zf,ease:Tf,easeVarying:Ff,end:sc,[Symbol.iterator]:ot[Symbol.iterator]};function cc(t){return((t*=2)<=1?t*t*t:(t-=2)*t*t+2)/2}var lc={time:null,delay:0,duration:250,ease:cc};function hc(t,n){for(var e;!(e=t.__transition)||!(e=e[n]);)if(!(t=t.parentNode))throw new Error(`transition ${n} not found`);return e}function dc(t){var n,e;t instanceof at?(n=t._id,t=t._name):(n=so(),(e=lc).time=Ar(),t=t==null?null:t+"");for(var r=this._groups,i=r.length,o=0;o<i;++o)for(var a=r[o],u=a.length,f,s=0;s<u;++s)(f=a[s])&&We(f,t,n,s,a,e||hc(f,n));return new at(r,this._parents,t,n)}Sn.prototype.interrupt=cf;Sn.prototype.transition=dc;function _n(t,n,e){this.k=t,this.x=n,this.y=e}_n.prototype={constructor:_n,scale:function(t){return t===1?this:new _n(this.k*t,this.x,this.y)},translate:function(t,n){return t===0&n===0?this:new _n(this.k,this.x+this.k*t,this.y+this.k*n)},apply:function(t){return[t[0]*this.k+this.x,t[1]*this.k+this.y]},applyX:function(t){return t*this.k+this.x},applyY:function(t){return t*this.k+this.y},invert:function(t){return[(t[0]-this.x)/this.k,(t[1]-this.y)/this.k]},invertX:function(t){return(t-this.x)/this.k},invertY:function(t){return(t-this.y)/this.k},rescaleX:function(t){return t.copy().domain(t.range().map(this.invertX,this).map(t.invert,t))},rescaleY:function(t){return t.copy().domain(t.range().map(this.invertY,this).map(t.invert,t))},toString:function(){return"translate("+this.x+","+this.y+") scale("+this.k+")"}};var Oc=new _n(1,0,0);function Hc(t){for(var n=t.length/6|0,e=new Array(n),r=0;r<n;)e[r]="#"+t.slice(r*6,++r*6);return e}function gc(t){return t===0?!0:t}function gt(t){return gc(t)?Array.isArray(t)?n=>t.map(e=>typeof e!="function"?n[e]:e(n)):typeof t!="function"?n=>n[t]:t:null}function yc(t){const n={};for(const e of t){if(Object(e)!==e)throw new TypeError("iterable for fromEntries should yield objects");const{"0":r,"1":i}=e;Object.defineProperty(n,r,{configurable:!0,enumerable:!0,writable:!0,value:i})}return n}function er(t,n={}){return yc(Object.entries(t).filter(([e,r])=>r!==void 0&&n[e]===void 0))}function mc(t,n){if(!Array.isArray(t))throw new TypeError("The first argument of calcExtents() must be an array. If you got this error using the <LayerCake> component, consider passing a flat array to the `flatData` prop. More info: https://layercake.graphics/guide/#flatdata");if(Array.isArray(n)||n===void 0||n===null)throw new TypeError("The second argument of calcExtents() must be an object with field names as keys as accessor functions as values.");const e={},r=Object.keys(n),i=r.length;let o,a,u,f,s,l,h,d;const y=t.length;for(o=0;o<i;o+=1){for(f=r[o],h=n[f],s=null,l=null,a=0;a<y;a+=1)if(d=h(t[a]),Array.isArray(d)){const m=d.length;for(u=0;u<m;u+=1)d[u]!==void 0&&d[u]!==null&&Number.isNaN(d[u])===!1&&((s===null||d[u]<s)&&(s=d[u]),(l===null||d[u]>l)&&(l=d[u]))}else d!=null&&Number.isNaN(d)===!1&&((s===null||d<s)&&(s=d),(l===null||d>l)&&(l=d));e[f]=[s,l]}return e}function fo(t=[],n){return Array.isArray(n)===!0?n.map((e,r)=>e===null?t[r]:e):t}function pe(t){return function([e,r]){return e?fo(e[t],r):r}}var wn={x:Ae,y:Ae,z:Ae,r:pu};function _c(t){return t.constant?"symlog":t.base?"log":t.exponent?t.exponent()===.5?"sqrt":"pow":"other"}function yi(t){return t}function wc(t){return n=>Math.log(t*n)}function pc(t){return n=>t*Math.exp(n)}function xc(t){return n=>Math.sign(n)*Math.log1p(Math.abs(n/t))}function vc(t){return n=>Math.sign(n)*Math.expm1(Math.abs(n))*t}function xe(t){return function(e){return e<0?-Math.pow(-e,t):Math.pow(e,t)}}function bc(t){const n=_c(t);if(n==="log"){const e=Math.sign(t.domain()[0]);return{lift:wc(e),ground:pc(e),scaleType:n}}return n==="pow"?{lift:xe(1),ground:xe(1/1),scaleType:n}:n==="sqrt"?{lift:xe(.5),ground:xe(1/.5),scaleType:n}:n==="symlog"?{lift:xc(1),ground:vc(1),scaleType:n}:{lift:yi,ground:yi,scaleType:n}}function Nc(t,n){if(typeof t.range!="function")throw new Error("Scale method `range` must be a function");if(typeof t.domain!="function")throw new Error("Scale method `domain` must be a function");if(!Array.isArray(n))return t.domain();t.domain().length!==2&&console.warn("[LayerCake] The scale is expected to have a domain of length 2 to use padding. Are you sure you want to use padding? Your scale's domain is:",t.domain()),t.range().length!==2&&console.warn("[LayerCake] The scale is expected to have a range of length 2 to use padding. Are you sure you want to use padding? Your scale's range is:",t.range());const{lift:e,ground:r}=bc(t),i=t.domain()[0],o=Object.prototype.toString.call(i)==="[object Date]",[a,u]=t.domain().map(y=>e(o?y.getTime():y)),[f,s]=t.range(),l=n[0]||0,h=n[1]||0,d=(u-a)/(Math.abs(s-f)-l-h);return[a-l*d,h*d+u].map(y=>r(o?new Date(y):y))}function kc(t,n,e,r,i){let o,a;return i===!0?(o=0,a=100):(o=t==="r"?1:0,a=t==="y"?e:t==="r"?25:n),r===!0?[a,o]:[o,a]}function Ac(t,n,e,r,i,o){return i?typeof i=="function"?i({width:n,height:e}):i:kc(t,n,e,r,o)}function ve(t){return function([e,r,i,o,a,u,f,s,l,h]){if(r===null)return null;const d=Ac(t,f,s,u,l,h),y=e===wn[t]?e():e.copy();return y.domain(fo(r[t],i)).range(d),o&&y.domain(Nc(y,o)),a===!0&&(typeof y.nice=="function"?y.nice():console.error(`[Layer Cake] You set \`${t}Nice: true\` but the ${t}Scale does not have a \`.nice\` method. Ignoring...`)),y}}function be([t,n]){return e=>{const r=t(e);return Array.isArray(r)?r.map(i=>n(i)):n(r)}}function Ne([t]){if(typeof t=="function"){if(typeof t.range=="function")return t.range();console.error("[LayerCake] Your scale doesn't have a `.range` method?")}return null}var ke={x:!1,y:!0,z:!1,r:!1};const Sc=t=>({element:t[0]&4,width:t[0]&256,height:t[0]&512,aspectRatio:t[0]&1024,containerWidth:t[0]&128,containerHeight:t[0]&64}),mi=t=>({element:t[2],width:t[8],height:t[9],aspectRatio:t[10],containerWidth:t[7],containerHeight:t[6]});function _i(t){let n,e,r;const i=t[90].default,o=vi(i,t,t[89],mi);return{c(){n=Re("div"),o&&o.c(),this.h()},l(a){n=Ee(a,"DIV",{class:!0});var u=_r(n);o&&o.l(u),u.forEach(vt),this.h()},h(){B(n,"class","layercake-container svelte-1ll0j6t"),Co(()=>t[92].call(n)),L(n,"position",t[5],!1),L(n,"top",t[5]==="absolute"?"0":null,!1),L(n,"right",t[5]==="absolute"?"0":null,!1),L(n,"bottom",t[5]==="absolute"?"0":null,!1),L(n,"left",t[5]==="absolute"?"0":null,!1),L(n,"pointer-events",t[4]===!1?"none":null,!1)},m(a,u){Ye(a,n,u),o&&o.m(n,null),t[91](n),e=To(n,t[92].bind(n)),r=!0},p(a,u){o&&o.p&&(!r||u[0]&1988|u[2]&134217728)&&bi(o,i,a,a[89],r?ki(i,a[89],u,Sc):Ni(a[89]),mi),u[0]&32&&L(n,"position",a[5],!1),u[0]&32&&L(n,"top",a[5]==="absolute"?"0":null,!1),u[0]&32&&L(n,"right",a[5]==="absolute"?"0":null,!1),u[0]&32&&L(n,"bottom",a[5]==="absolute"?"0":null,!1),u[0]&32&&L(n,"left",a[5]==="absolute"?"0":null,!1),u[0]&16&&L(n,"pointer-events",a[4]===!1?"none":null,!1)},i(a){r||(pn(o,a),r=!0)},o(a){ze(o,a),r=!1},d(a){a&&vt(n),o&&o.d(a),t[91](null),e()}}}function Mc(t){let n,e,r=(t[3]===!0||typeof window!="undefined")&&_i(t);return{c(){r&&r.c(),n=Lr()},l(i){r&&r.l(i),n=Lr()},m(i,o){r&&r.m(i,o),Ye(i,n,o),e=!0},p(i,o){i[3]===!0||typeof window!="undefined"?r?(r.p(i,o),o[0]&8&&pn(r,1)):(r=_i(i),r.c(),pn(r,1),r.m(n.parentNode,n)):r&&(Po(),ze(r,1,1,()=>{r=null}),Fo())},i(i){e||(pn(r),e=!0)},o(i){ze(r),e=!1},d(i){r&&r.d(i),i&&vt(n)}}}function Dc(t,n,e){let r,i,o,a,u,f,s,l,h,d,y,m,v,M,T,E,A,k,b,z,yt,K,mt,ut,w,F,R,H,st,U,q,J,C,ft,Z,_t,wt,Jt,Mn,g,X,{$$slots:I={},$$scope:Y}=n,{ssr:et=!1}=n,{pointerEvents:Dn=!0}=n,{position:Rn="relative"}=n,{percentRange:kt=!1}=n,{width:Zt=void 0}=n,{height:Qt=void 0}=n,{containerWidth:ct=Zt||100}=n,{containerHeight:At=Qt||100}=n,{element:En=void 0}=n,{x:St=void 0}=n,{y:Mt=void 0}=n,{z:Dt=void 0}=n,{r:Rt=void 0}=n,{data:Et=[]}=n,{xDomain:zt=void 0}=n,{yDomain:Ct=void 0}=n,{zDomain:Tt=void 0}=n,{rDomain:Pt=void 0}=n,{xNice:zn=!1}=n,{yNice:Cn=!1}=n,{zNice:Tn=!1}=n,{rNice:Pn=!1}=n,{xReverse:Fn=ke.x}=n,{yReverse:In=ke.y}=n,{zReverse:Ln=ke.z}=n,{rReverse:On=ke.r}=n,{xPadding:Hn=void 0}=n,{yPadding:qn=void 0}=n,{zPadding:Xn=void 0}=n,{rPadding:Yn=void 0}=n,{xScale:Bn=wn.x}=n,{yScale:Gn=wn.y}=n,{zScale:Vn=wn.z}=n,{rScale:Wn=wn.r}=n,{xRange:Ft=void 0}=n,{yRange:It=void 0}=n,{zRange:Lt=void 0}=n,{rRange:Ot=void 0}=n,{padding:Kn={}}=n,{extents:Un={}}=n,{flatData:Jn=void 0}=n,{custom:Zn={}}=n,Ke=!1;Ai(()=>{Ke=!0});const N={},lt=x(kt);_(t,lt,c=>e(128,Jt=c));const Ht=x(ct);_(t,Ht,c=>e(7,wt=c));const qt=x(At);_(t,qt,c=>e(6,_t=c));const Qn=x(er(Un));_(t,Qn,c=>e(127,Z=c));const jn=x(Et);_(t,jn,c=>e(126,ft=c));const jt=x(Jn||Et);_(t,jt,c=>e(125,C=c));const $n=x(Kn);_(t,$n,c=>e(124,J=c));const Xt=x(gt(St));_(t,Xt,c=>e(123,q=c));const Yt=x(gt(Mt));_(t,Yt,c=>e(122,U=c));const Bt=x(gt(Dt));_(t,Bt,c=>e(121,st=c));const Gt=x(gt(Rt));_(t,Gt,c=>e(120,H=c));const te=x(zt);_(t,te,c=>e(119,R=c));const ne=x(Ct);_(t,ne,c=>e(118,F=c));const ee=x(Tt);_(t,ee,c=>e(117,w=c));const re=x(Pt);_(t,re,c=>e(116,ut=c));const $t=x(zn);_(t,$t,c=>e(115,mt=c));const tn=x(Cn);_(t,tn,c=>e(114,K=c));const nn=x(Tn);_(t,nn,c=>e(113,yt=c));const en=x(Pn);_(t,en,c=>e(112,z=c));const rn=x(Fn);_(t,rn,c=>e(111,b=c));const on=x(In);_(t,on,c=>e(110,k=c));const an=x(Ln);_(t,an,c=>e(109,A=c));const un=x(On);_(t,un,c=>e(108,E=c));const sn=x(Hn);_(t,sn,c=>e(107,T=c));const fn=x(qn);_(t,fn,c=>e(106,M=c));const cn=x(Xn);_(t,cn,c=>e(105,v=c));const ln=x(Yn);_(t,ln,c=>e(104,m=c));const ie=x(Ft);_(t,ie,c=>e(103,y=c));const oe=x(It);_(t,oe,c=>e(102,d=c));const ae=x(Lt);_(t,ae,c=>e(101,h=c));const ue=x(Ot);_(t,ue,c=>e(100,l=c));const se=x(Bn);_(t,se,c=>e(99,s=c));const fe=x(Gn);_(t,fe,c=>e(98,f=c));const ce=x(Vn);_(t,ce,c=>e(97,u=c));const le=x(Wn);_(t,le,c=>e(96,a=c));const he=x(N);_(t,he,c=>e(94,i=c));const de=x(Zn);_(t,de,c=>e(95,o=c));const Dr=S([Xt,Yt,Bt,Gt],([c,it,Q,D])=>{const hn={};return c&&(hn.x=c),it&&(hn.y=it),Q&&(hn.z=Q),D&&(hn.r=D),hn}),Rr=S([$n,Ht,qt],([c])=>Object.assign({top:0,right:0,bottom:0,left:0},c)),Er=S([Ht,qt,Rr],([c,it,Q])=>{const D={};return D.top=Q.top,D.right=c-Q.right,D.bottom=it-Q.bottom,D.left=Q.left,D.width=D.right-D.left,D.height=D.bottom-D.top,D.width<=0&&Ke===!0&&console.warn("[LayerCake] Target div has zero or negative width. Did you forget to set an explicit width in CSS on the container?"),D.height<=0&&Ke===!0&&console.warn("[LayerCake] Target div has zero or negative height. Did you forget to set an explicit height in CSS on the container?"),D}),ht=S([Er],([c])=>c.width);_(t,ht,c=>e(8,Mn=c));const dt=S([Er],([c])=>c.height);_(t,dt,c=>e(9,g=c));const rt=S([jt,Dr,Qn],([c,it,Q])=>{const D=er(it,Q);return Object.keys(D).length>0?$e($e({},mc(c,D)),Q):{}}),zr=S([rt,te],pe("x")),Cr=S([rt,ne],pe("y")),Tr=S([rt,ee],pe("z")),Pr=S([rt,re],pe("r")),Ue=S([se,rt,zr,sn,$t,rn,ht,dt,ie,lt],ve("x")),co=S([Xt,Ue],be),Je=S([fe,rt,Cr,fn,tn,on,ht,dt,oe,lt],ve("y")),lo=S([Yt,Je],be),Ze=S([ce,rt,Tr,cn,nn,an,ht,dt,ae,lt],ve("z")),ho=S([Bt,Ze],be),Qe=S([le,rt,Pr,ln,en,un,ht,dt,ue,lt],ve("r")),go=S([Gt,Qe],be),yo=S([Ue],Ne),mo=S([Je],Ne),_o=S([Ze],Ne),wo=S([Qe],Ne),je=S([ht,dt],([c,it])=>c/it);_(t,je,c=>e(10,X=c));function po(c){ir[c?"unshift":"push"](()=>{En=c,e(2,En)})}function xo(){ct=this.clientWidth,At=this.clientHeight,e(0,ct),e(1,At)}return t.$$set=c=>{"ssr"in c&&e(3,et=c.ssr),"pointerEvents"in c&&e(4,Dn=c.pointerEvents),"position"in c&&e(5,Rn=c.position),"percentRange"in c&&e(51,kt=c.percentRange),"width"in c&&e(52,Zt=c.width),"height"in c&&e(53,Qt=c.height),"containerWidth"in c&&e(0,ct=c.containerWidth),"containerHeight"in c&&e(1,At=c.containerHeight),"element"in c&&e(2,En=c.element),"x"in c&&e(54,St=c.x),"y"in c&&e(55,Mt=c.y),"z"in c&&e(56,Dt=c.z),"r"in c&&e(57,Rt=c.r),"data"in c&&e(58,Et=c.data),"xDomain"in c&&e(59,zt=c.xDomain),"yDomain"in c&&e(60,Ct=c.yDomain),"zDomain"in c&&e(61,Tt=c.zDomain),"rDomain"in c&&e(62,Pt=c.rDomain),"xNice"in c&&e(63,zn=c.xNice),"yNice"in c&&e(64,Cn=c.yNice),"zNice"in c&&e(65,Tn=c.zNice),"rNice"in c&&e(66,Pn=c.rNice),"xReverse"in c&&e(67,Fn=c.xReverse),"yReverse"in c&&e(68,In=c.yReverse),"zReverse"in c&&e(69,Ln=c.zReverse),"rReverse"in c&&e(70,On=c.rReverse),"xPadding"in c&&e(71,Hn=c.xPadding),"yPadding"in c&&e(72,qn=c.yPadding),"zPadding"in c&&e(73,Xn=c.zPadding),"rPadding"in c&&e(74,Yn=c.rPadding),"xScale"in c&&e(75,Bn=c.xScale),"yScale"in c&&e(76,Gn=c.yScale),"zScale"in c&&e(77,Vn=c.zScale),"rScale"in c&&e(78,Wn=c.rScale),"xRange"in c&&e(79,Ft=c.xRange),"yRange"in c&&e(80,It=c.yRange),"zRange"in c&&e(81,Lt=c.zRange),"rRange"in c&&e(82,Ot=c.rRange),"padding"in c&&e(83,Kn=c.padding),"extents"in c&&e(84,Un=c.extents),"flatData"in c&&e(85,Jn=c.flatData),"custom"in c&&e(86,Zn=c.custom),"$$scope"in c&&e(89,Y=c.$$scope)},t.$$.update=()=>{t.$$.dirty[1]&8388608&&St&&e(87,N.x=St,N),t.$$.dirty[1]&16777216&&Mt&&e(87,N.y=Mt,N),t.$$.dirty[1]&33554432&&Dt&&e(87,N.z=Dt,N),t.$$.dirty[1]&67108864&&Rt&&e(87,N.r=Rt,N),t.$$.dirty[1]&268435456&&zt&&e(87,N.xDomain=zt,N),t.$$.dirty[1]&536870912&&Ct&&e(87,N.yDomain=Ct,N),t.$$.dirty[1]&1073741824&&Tt&&e(87,N.zDomain=Tt,N),t.$$.dirty[2]&1&&Pt&&e(87,N.rDomain=Pt,N),t.$$.dirty[2]&131072&&Ft&&e(87,N.xRange=Ft,N),t.$$.dirty[2]&262144&&It&&e(87,N.yRange=It,N),t.$$.dirty[2]&524288&&Lt&&e(87,N.zRange=Lt,N),t.$$.dirty[2]&1048576&&Ot&&e(87,N.rRange=Ot,N),t.$$.dirty[1]&1048576&&p(lt,Jt=kt,Jt),t.$$.dirty[0]&1&&p(Ht,wt=ct,wt),t.$$.dirty[0]&2&&p(qt,_t=At,_t),t.$$.dirty[2]&4194304&&p(Qn,Z=er(Un),Z),t.$$.dirty[1]&134217728&&p(jn,ft=Et,ft),t.$$.dirty[1]&134217728|t.$$.dirty[2]&8388608&&p(jt,C=Jn||Et,C),t.$$.dirty[2]&2097152&&p($n,J=Kn,J),t.$$.dirty[1]&8388608&&p(Xt,q=gt(St),q),t.$$.dirty[1]&16777216&&p(Yt,U=gt(Mt),U),t.$$.dirty[1]&33554432&&p(Bt,st=gt(Dt),st),t.$$.dirty[1]&67108864&&p(Gt,H=gt(Rt),H),t.$$.dirty[1]&268435456&&p(te,R=zt,R),t.$$.dirty[1]&536870912&&p(ne,F=Ct,F),t.$$.dirty[1]&1073741824&&p(ee,w=Tt,w),t.$$.dirty[2]&1&&p(re,ut=Pt,ut),t.$$.dirty[2]&2&&p($t,mt=zn,mt),t.$$.dirty[2]&4&&p(tn,K=Cn,K),t.$$.dirty[2]&8&&p(nn,yt=Tn,yt),t.$$.dirty[2]&16&&p(en,z=Pn,z),t.$$.dirty[2]&32&&p(rn,b=Fn,b),t.$$.dirty[2]&64&&p(on,k=In,k),t.$$.dirty[2]&128&&p(an,A=Ln,A),t.$$.dirty[2]&256&&p(un,E=On,E),t.$$.dirty[2]&512&&p(sn,T=Hn,T),t.$$.dirty[2]&1024&&p(fn,M=qn,M),t.$$.dirty[2]&2048&&p(cn,v=Xn,v),t.$$.dirty[2]&4096&&p(ln,m=Yn,m),t.$$.dirty[2]&131072&&p(ie,y=Ft,y),t.$$.dirty[2]&262144&&p(oe,d=It,d),t.$$.dirty[2]&524288&&p(ae,h=Lt,h),t.$$.dirty[2]&1048576&&p(ue,l=Ot,l),t.$$.dirty[2]&8192&&p(se,s=Bn,s),t.$$.dirty[2]&16384&&p(fe,f=Gn,f),t.$$.dirty[2]&32768&&p(ce,u=Vn,u),t.$$.dirty[2]&65536&&p(le,a=Wn,a),t.$$.dirty[2]&16777216&&p(de,o=Zn,o),t.$$.dirty[2]&33554432&&p(he,i=N,i),t.$$.dirty[2]&67108864&&Io("LayerCake",r)},e(88,r={activeGetters:Dr,width:ht,height:dt,percentRange:lt,aspectRatio:je,containerWidth:Ht,containerHeight:qt,x:Xt,y:Yt,z:Bt,r:Gt,custom:de,data:jn,xNice:$t,yNice:tn,zNice:nn,rNice:en,xReverse:rn,yReverse:on,zReverse:an,rReverse:un,xPadding:sn,yPadding:fn,zPadding:cn,rPadding:ln,padding:Rr,flatData:jt,extents:rt,xDomain:zr,yDomain:Cr,zDomain:Tr,rDomain:Pr,xRange:yo,yRange:mo,zRange:_o,rRange:wo,config:he,xScale:Ue,xGet:co,yScale:Je,yGet:lo,zScale:Ze,zGet:ho,rScale:Qe,rGet:go}),[ct,At,En,et,Dn,Rn,_t,wt,Mn,g,X,lt,Ht,qt,Qn,jn,jt,$n,Xt,Yt,Bt,Gt,te,ne,ee,re,$t,tn,nn,en,rn,on,an,un,sn,fn,cn,ln,ie,oe,ae,ue,se,fe,ce,le,he,de,ht,dt,je,kt,Zt,Qt,St,Mt,Dt,Rt,Et,zt,Ct,Tt,Pt,zn,Cn,Tn,Pn,Fn,In,Ln,On,Hn,qn,Xn,Yn,Bn,Gn,Vn,Wn,Ft,It,Lt,Ot,Kn,Un,Jn,Zn,N,r,Y,I,po,xo]}class qc extends wi{constructor(n){super(),pi(this,n,Dc,Mc,xi,{ssr:3,pointerEvents:4,position:5,percentRange:51,width:52,height:53,containerWidth:0,containerHeight:1,element:2,x:54,y:55,z:56,r:57,data:58,xDomain:59,yDomain:60,zDomain:61,rDomain:62,xNice:63,yNice:64,zNice:65,rNice:66,xReverse:67,yReverse:68,zReverse:69,rReverse:70,xPadding:71,yPadding:72,zPadding:73,rPadding:74,xScale:75,yScale:76,zScale:77,rScale:78,xRange:79,yRange:80,zRange:81,rRange:82,padding:83,extents:84,flatData:85,custom:86},null,[-1,-1,-1,-1,-1])}}export{Tc as D,qc as L,O as S,_n as T,Mu as a,ff as b,Hc as c,Bi as d,ar as e,wr as f,Xi as g,$ as h,Oc as i,Fc as j,Ka as k,Ae as l,vr as m,Pc as n,va as o,Ic as p,Hi as q,no as r,Lc as s,io as t};
