import{S as ve,i as fe,s as ne,e as r,t as he,k as C,c as l,a as t,h as me,d as a,m as W,b as e,$ as N,g as pe,I as s,E as z}from"../chunks/index-d6b53ebe.js";function be(ue){let o,h,i,G,O,P,m,p,b,c,_,g,x,u,B,R,w,E,I,v,F,T,D,V,j,f,H,$,k,y,A,n,J;return{c(){o=r("div"),h=r("div"),i=r("div"),G=r("p"),O=he("Welcome to the helics-cli web interface."),P=C(),m=r("div"),p=r("section"),b=r("div"),c=r("div"),_=r("div"),g=r("div"),x=r("a"),u=r("img"),R=C(),w=r("div"),E=r("div"),I=r("a"),v=r("img"),T=C(),D=r("div"),V=r("div"),j=r("a"),f=r("img"),$=C(),k=r("div"),y=r("div"),A=r("a"),n=r("img"),this.h()},l(M){o=l(M,"DIV",{class:!0});var S=t(o);h=l(S,"DIV",{class:!0});var K=t(h);i=l(K,"DIV",{class:!0});var q=t(i);G=l(q,"P",{});var L=t(G);O=me(L,"Welcome to the helics-cli web interface."),L.forEach(a),P=W(q),m=l(q,"DIV",{class:!0});var Q=t(m);p=l(Q,"SECTION",{class:!0});var U=t(p);b=l(U,"DIV",{class:!0});var X=t(b);c=l(X,"DIV",{class:!0});var d=t(c);_=l(d,"DIV",{class:!0});var Y=t(_);g=l(Y,"DIV",{class:!0});var Z=t(g);x=l(Z,"A",{href:!0});var ee=t(x);u=l(ee,"IMG",{alt:!0,class:!0,src:!0}),ee.forEach(a),Z.forEach(a),Y.forEach(a),R=W(d),w=l(d,"DIV",{class:!0});var se=t(w);E=l(se,"DIV",{class:!0});var re=t(E);I=l(re,"A",{href:!0});var le=t(I);v=l(le,"IMG",{alt:!0,class:!0,src:!0}),le.forEach(a),re.forEach(a),se.forEach(a),T=W(d),D=l(d,"DIV",{class:!0});var ae=t(D);V=l(ae,"DIV",{class:!0});var te=t(V);j=l(te,"A",{href:!0});var ce=t(j);f=l(ce,"IMG",{alt:!0,class:!0,src:!0}),ce.forEach(a),te.forEach(a),ae.forEach(a),$=W(d),k=l(d,"DIV",{class:!0});var oe=t(k);y=l(oe,"DIV",{class:!0});var ie=t(y);A=l(ie,"A",{href:!0});var de=t(A);n=l(de,"IMG",{alt:!0,class:!0,src:!0}),de.forEach(a),ie.forEach(a),oe.forEach(a),d.forEach(a),X.forEach(a),U.forEach(a),Q.forEach(a),q.forEach(a),K.forEach(a),S.forEach(a),this.h()},h(){e(u,"alt","observer"),e(u,"class","block object-cover object-center w-full h-full rounded-lg"),N(u.src,B="https://user-images.githubusercontent.com/1813121/179559286-10a9b8bb-cccd-448c-9980-851d396c256c.png")||e(u,"src",B),e(x,"href","/observe"),e(g,"class","w-full p-1 md:p-2 border"),e(_,"class","flex flex-wrap w-1/5"),e(v,"alt","profiler"),e(v,"class","block object-cover object-center w-full h-full rounded-lg"),N(v.src,F="https://user-images.githubusercontent.com/1813121/179559394-5ca86ce6-0b9c-412b-9bbb-657fa16c72f3.png")||e(v,"src",F),e(I,"href","/profile"),e(E,"class","w-full p-1 md:p-2 border"),e(w,"class","flex flex-wrap w-1/5"),e(f,"alt","broker"),e(f,"class","block object-cover object-center w-full h-full rounded-lg"),N(f.src,H="https://user-images.githubusercontent.com/1813121/179560242-74c58446-098a-4890-85bc-805d39940f2f.png")||e(f,"src",H),e(j,"href","/broker"),e(V,"class","w-full p-1 md:p-2 border"),e(D,"class","flex flex-wrap w-1/5"),e(n,"alt","runner"),e(n,"class","block object-cover object-center w-full h-full rounded-lg"),N(n.src,J="https://user-images.githubusercontent.com/1813121/179692080-b4dc172e-c73e-4982-8d97-17051dca8c26.png")||e(n,"src",J),e(A,"href","/run"),e(y,"class","w-full p-1 md:p-2 border"),e(k,"class","flex flex-wrap w-1/5"),e(c,"class","flex flex-wrap -m-1 md:-m-2 space-x-4"),e(b,"class","container px-5 py-2 mx-auto lg:pt-12 lg:px-32"),e(p,"class","overflow-hidden text-gray-700 "),e(m,"class","flex space-x-4"),e(i,"class","py-2 inline-block w-full sm:px-6 lg:px-8"),e(h,"class","overflow-x-auto sm:-mx-6 lg:-mx-8"),e(o,"class","flex w-7/8 flex-col mt-6 mx-8")},m(M,S){pe(M,o,S),s(o,h),s(h,i),s(i,G),s(G,O),s(i,P),s(i,m),s(m,p),s(p,b),s(b,c),s(c,_),s(_,g),s(g,x),s(x,u),s(c,R),s(c,w),s(w,E),s(E,I),s(I,v),s(c,T),s(c,D),s(D,V),s(V,j),s(j,f),s(c,$),s(c,k),s(k,y),s(y,A),s(A,n)},p:z,i:z,o:z,d(M){M&&a(o)}}}class ge extends ve{constructor(o){super(),fe(this,o,null,be,ne,{})}}export{ge as default};
