canvas.svelte-zs93ed{float:left}.layercake-container.svelte-1ll0j6t,.layercake-container.svelte-1ll0j6t *{box-sizing:border-box}.layercake-container.svelte-1ll0j6t{width:100%;height:100%}div.svelte-1u9nfei,slot.svelte-1u9nfei{position:absolute;top:0;left:0}svg.svelte-17l4rdd{position:absolute;top:0;left:0;overflow:visible}svg.svelte-ocozda{position:absolute;width:100%;height:100%;overflow:visible}svg.svelte-ocozda *{vector-effect:non-scaling-stroke}
