import{w as l}from"./index-e4f78b56.js";import{_ as n}from"./index-d6b53ebe.js";const i=(r,a)=>{const e=l(a),s=localStorage.getItem(r);return s!=null&&e.set(JSON.parse(s)),e.subscribe(t=>{[null,void 0].includes(t)?localStorage.removeItem(r):localStorage.setItem(r,JSON.stringify(t))}),window.addEventListener("storage",()=>{const t=localStorage.getItem(r);if(t==null)return;const o=JSON.parse(t);o!==n(e)&&e.set(o)}),e},f={profile:[],runner:{broker:!1,federates:[],name:"web",path:null},federates:[],cores:[],systeminfo:{},graphs:{},publications:[],inputs:[],table:[],table_columns:[],cores_columns:[{field:"id",sortName:"id",title:"ID",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"name",sortName:"name",title:"Name",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"address",sortName:"address",title:"Address",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1}],federates_columns:[{field:"id",sortName:"id",title:"ID",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"name",sortName:"name",title:"Name",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"core_name",sortName:"core_name",title:"Core Name",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1}],inputs_columns:[{field:"id",sortName:"id",title:"ID",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"source_name",sortName:"source_name",title:"Source",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"target_name",sortName:"target_name",title:"Target",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1}],publications_columns:[{field:"id",sortName:"id",title:"ID",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"name",sortName:"name",title:"Name",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"source_name",sortName:"source_name",title:"Source",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1},{field:"target_name",sortName:"target_name",title:"Target",show:!0,sort:"ascending",sortColor:"#fff",sortActive:!0,text_center:!1}]},d=i("data",f);export{f as D,d};
