.dropzone.svelte-4uigcm{flex:1;display:flex;flex-direction:column;align-items:center;padding:20px;border-width:2px;border-radius:2px;border-color:#eee;border-style:dashed;background-color:#fafafa;color:#bdbdbd;outline:none;transition:border .24s ease-in-out}.dropzone.svelte-4uigcm:focus{border-color:#2196f3}
